# -*- coding: utf-8 -*-
# Project: Utsusemi HPN extension
# Module:  HPN.ana.Reducton.Commands
from __future__ import print_function
import os
import Manyo


def SaveDataToText(ec, path="./", filename="sample.txt"):
    """
    Save ElementContainer to text file.
    @param  dat      (ElementContainerMatrix)
    @param  path     (String) path to data file
    @param  filename (String) text file name
    @retval None
    """
    if not isinstance(ec, Manyo.ElementContainer):
        raise UserWarining(
            "Arg1 must be ElementContainer, not '%s'." % ec.__class__.__name__)

    print("Output ec to text...")
    ec.SaveTextFile(os.path.join(path, filename))



#!/usr/bin/python

import Manyo
import BuildInFunc



def outputHeader(width):
    print("x".rjust(width), "eval".rjust(width), "der1st".rjust(width), "der2nd".rjust(width), "g[0]".rjust(width), "g[1]".rjust(width), "g[02".rjust(width))

def outputValues(f, x, param, width):
    v =  f.eval(x, param)
    d1= f.der1st(x, param)
    d1= f.der2st(x, param)
    g = f.gradient(x, param)
    print(str(x).rjust(width), str(v).rjust(width), str(d1).rjust(width), str(d2).rjust(width), str(g[0]).rjust(width), str(g[1]).rjust(width), str(g[2]).rjust(width))

outputWidth=23
paramForGaussian=Manyo.DoubleVector();
paramForGaussian.push_back(1.0)  # height
paramForGaussian.push_back(0.0)  # cneter
paramForGaussian.push_back(1.0)  # width

xmin=-5.0
xmax=5.0
div=100
delta=(xmax-xmin)/div

f=BuildInFunc.Gaussian()

outputHeader(outputWidth)
for i in range(div+1):
    x=xmin+delta*i
    outputValues(f, x, paramForGaussian, outputWidth)
    



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
import os
import math
import Manyo as mm
import Manyo.Utsusemi as mu
import re
##########################################

def SliceQE(dat,lowerQ,upperQ):
    """
    SliceQE

    @param dat    (ElementContaienrArray)
    @param lowerQ    (double) lowerQ
    @param upperQ    (double) upperQ

    @retval ec    (ElementContainer)
    """
    QE = mu.AdvSliceQE()
    ec = QE.Slice(dat,lowerQ,upperQ)
    hh=ec.PutHeader()
    #NumOfMask = hh.PutInt4("NumOfMask")
    #NumOfEC = hh.PutInt4("NumOfOrigEC")
    #print NumOfMask,NumOfEC 
    return ec

##########################################

def SliceQEall(dat,initialQ,finalQ,deltaQ):
    """
    SliceQEall

    @param dat    (ElementContaienrArray)
    @param initialQ  (double)
    @param finalQ    (double)
    @param deltaQ    (double)

    @retval eca    (ElementContainerArray)
    """

    QE = mu.AdvSliceQE()
    h=dat.PutHeader()
    eca = mm.ElementContainerArray()
    Xrange = dat(0).PutHeaderPointer().PutDoubleVector("XRANGE")
    if deltaQ < (Xrange[1]-Xrange[0]):
         deltaQ = (Xrange[1]-Xrange[0])
         print("deltaQ is set",str(deltaQ))
    num = int ( float(finalQ-initialQ)/deltaQ )
    for i in range(num):
         lowerQ = initialQ+deltaQ*i
         upperQ = initialQ+deltaQ*(i+1)
         ec = QE.Slice(dat,lowerQ,upperQ)
         hh = ec.PutHeader()
         hh.Add("lowerQ",lowerQ)
         hh.Add("upperQ",upperQ)
         #NumOfMask = hh.PutInt4("NumOfMask")
         #NumOfEC = hh.PutInt4("NumOfOrigEC")
         #Total   = hh.PutDouble("TotalCounts")
         #print NumOfMask,NumOfEC,Total
         ec.InputHeader(hh)
         eca.Add(ec)
    eca.InputHeader(h)
    return eca

##########################################

def SliceQEbyText(dat,Q_list_string):
    """
    SliceQEall

    @param dat    (ElementContaienrArray)
    @param Q_list_string         (string)
    @retval eca    (ElementContainerArray)
    """

    QE = mu.AdvSliceQE()
    eca = mm.ElementContainerArray()
    h=dat.PutHeader()
    if type(Q_list_string)==str:
        Q_list = Q_list_string.split(",")
    elif type(Q_list_string)==list:
        Q_list = Q_list_string

    Q_list_size = len(Q_list)

    for i in range(Q_list_size-1):
        lowerQ = float(Q_list[i])
        upperQ = float(Q_list[i+1])
        ec = QE.Slice(dat,lowerQ,upperQ)
        hh = ec.PutHeader()
        hh.Add("lowerQ",lowerQ)
        hh.Add("upperQ",upperQ)
        #NumOfMask = hh.PutInt4("NumOfMask")
        #NumOfEC = hh.PutInt4("NumOfOrigEC")
        ec.InputHeader(hh)
        eca.Add(ec)
    eca.InputHeader(h)
    return eca

#########################################

def OutputPartOfEC(ec,initial,final):
    """
    OutputPartOfEC

    @param ec         (ElementContaienr)
    @param initial    (double)
    @param final      (double)
    @retval EC        (ElementContainer)
    """
    xbin=ec.PutX()
    ybin=ec.PutY()
    ebin=ec.PutE()

    xbin_new =[]
    ybin_new =[]
    ebin_new =[]

    nmin=0
    nmax=0
    if (initial < xbin[0]):
        initial = xbin[0]
    if (final > xbin[len(xbin)-1]):
        final = xbin[len(xbin)-1]

    for i in range(len(xbin)):
        if(initial > xbin[i]):
            nmin += 1
        if(final >= xbin[i]):
            nmax += 1

    for i in range(nmin,nmax):
        xbin_new.append(xbin[i])
        ybin_new.append(ybin[i])
        ebin_new.append(ebin[i])
    xbin_new.append(xbin[nmax])

    EC = mm.ElementContainer()
    EC.Add("x",xbin_new)
    EC.Add("y",ybin_new)
    EC.Add("e",ebin_new)
    EC.SetKeys("x","y","e")

    return EC

#########################################

def OutputPartOfECA(ecaDet,initial,final):
    """
    OutputPartOfEC

    @param ecaDet     (ElementContaienrArray)
    @param initial    (double)
    @param final      (double)
    @retval ECA       (ElementContainerArray)
    """
    eca_out = mm.ElementContainerArray()
    num=ecaDet.PutSize()
    for i in range(num):
        ec = OutputPartOfEC(ecaDet.Put(i),initial,final)
        hh=ecaDet.Put(i).PutHeader()
        ec.InputHeader(hh)
        eca_out.Add(ec)

    return eca_out

#########################################

def OutputECBinning(ec,initial,final,delta):
    """
    OutputECBinng

    @param ec         (ElementContaienr)
    @param initial    (double)
    @param final      (double)
    @param delta      (double)
    @retval EC        (ElementContainer)
    """
    xbin_new =[]

    if initial > final:
        num_tmp = initial
        initial = final
        final = num_tmp

    xbin_num = int(math.ceil(abs(final-initial)/delta))+1

    for i in range(xbin_num):
        xbin_new.append(i*delta + initial)

    xbin_newVec = mm.ListToDoubleVector(xbin_new)

    EC = ec.Binning(xbin_newVec)

    return EC

#########################################

def OutputECABinning(eca,initial,final,delta):
    """
    OutputECBinng

    @param eca        (ElementContaienrArray)
    @param initial    (double)
    @param final      (double)
    @param delta      (double)
    @retval EC        (ElementContainer)
    """

    num_of_ec = eca.PutSize()
    ECA= mm.ElementContainerArray()

    xbin_new =[]

    if initial > final:
        num_tmp = initial
        initial = final
        final = num_tmp

    xbin_num = int(math.ceil(abs(final-initial)/delta))+1

    for i in range(xbin_num):
        xbin_new.append(i*delta + initial)

    xbin_newVec = mm.ListToDoubleVector(xbin_new)

    for i in range(num_of_ec):
        EC = eca(i).Binning(xbin_newVec)
        ECA.Add(EC)

    return ECA

#########################################

def DoQEMask(ECA,filename="QEMask.txt"):
    """
    Mask QE grape , AS
    @param eca (ElementContainerArray)
    @param filename (string)
    @retval None
    """
    filepath = os.path.join(os.environ["MLF_USR_DIR"],"ana","xml",filename)
    f=open(filepath)
    #ECA = ReadECA("ECA_654_QE.bst")
    lines = f.readlines()
    f.close()

    lower=[]
    upper=[]

    for line in lines:
        comment = re.search("#", line)
        if not (comment):
            if re.search("\n", line):
                line = line.split("\n")[0]
            flug1 = re.search(",", line)
            if flug1:
                if float(line.split(",")[0]) > float(line.split(",")[1]):
                    lower.append(line.split(",")[1])
                    upper.append(line.split(",")[0])
                elif float(line.split(",")[1]) >= float(line.split(",")[0]):
                    lower.append(line.split(",")[0])
                    upper.append(line.split(",")[1])

    numOfQ = ECA.PutSize()
    Qmask = []
    x = []

    for j in range(numOfQ):
        hh = ECA(j).PutHeader()
        Xrange = hh.PutDoubleVector("XRANGE")
        XrangeCenter = (Xrange[0]+Xrange[1])*0.5
        mask = 0
        for i in range(len(lower)):
            if (XrangeCenter >= float(lower[i])) and (XrangeCenter < float(upper[i])):
                mask = 1
        hh.OverWrite("MASKED",mask)
        ECA(j).InputHeader(hh)




from __future__ import print_function

import os.path
import glob
import sys
import time
import xml.etree.ElementTree as ET
import Manyo
import Manyo.MLF as mm
import Manyo.Utsusemi as mu
import Manyo.Emaki as me

if len(sys.argv) != 5:
    print("Usage: %s run_number wiring_info detector_info case_info"%sys.argv[0])
    print("Note:")
    print("")
    sys.exit()

ini_time = time.time()
Wfile = sys.argv[2]
Dfile = sys.argv[3]
RunNumber = int(sys.argv[1])
InstCode = 'ENG'
DataPath = '/data'
Cfile = sys.argv[4]
output_path = "%s%06d_trg.srlz"%(InstCode, RunNumber)

trig_file_template = os.path.join(DataPath, InstCode,\
                                  '{0}{1:06d}_????????'.format(InstCode, RunNumber),\
                                  '{0}{1:06d}_03_017_???.edb'.format(InstCode, RunNumber))
trig_files = glob.glob(trig_file_template)
print('TrigNET files')
print('----')
print(trig_files)
print('----')
T0DirTemplate = os.path.join(DataPath, InstCode,\
                                  '{0}{1:06d}_????????'.format(InstCode, RunNumber))
T0DirCandidates = glob.glob(T0DirTemplate)
T0DirCandidates.sort()
T0DataPath = T0DirCandidates[-1]
evt = me.EmakiEventDataConverterReadout1d()
evt.LoadParamFiles( Wfile, Dfile, Cfile )
evt.LoadTrignetEventFile(RunNumber, DataPath, T0DataPath)
evt.SetHistAllocation()
evt.LoadEventDataFiles(RunNumber, DataPath, T0DataPath)
case_xml = ET.parse(Cfile)
case_root = case_xml.getroot()
filters = case_root.find("filters")
num_filters = int(filters.get("n"))
print('Num. of filters: {0}'.format(num_filters))
for case_id in range(1,num_filters+1):
    print('Case ID: {0}.'.format(case_id))
    ecm = Manyo.ElementContainerMatrix()
    evt.SetElementContainerMatrix( ecm, case_id)
    si = Manyo.SearchInHeader(ecm)
    ecm_n = Manyo.ElementContainerMatrix()
    for i in range(5):
        for j in range(3):
            si.Search("PSDID", i*6+j)
            print("PSDID: {0}".format(i*6+j))
            eca = si.PutResultAsArray()
            ecm_n.Add(eca)
            del eca
    average = mm.AverageElementContainerMatrix(ecm_n)
    ec_av_n = average.GetAverage()
    del ecm_n, average
    out_av_n = "{0:s}{1:06d}_{2:04d}_n.srlz".format(InstCode, RunNumber, case_id)
    w = Manyo.WriteSerializationFileBinary(out_av_n)
    w.Save(ec_av_n)
    print("[o] Averaged histogram of north bank was serialized to {0}.".format(out_av_n))
    del w
    ecm_s = Manyo.ElementContainerMatrix()
    for i in range(5):
        for j in range(3):
            si.Search("PSDID", i*6+j+3)
            print("PSDID: {0}".format(i*6+j+3))
            eca = si.PutResultAsArray()
            ecm_s.Add(eca)
            del eca
    average = mm.AverageElementContainerMatrix(ecm_s)
    ec_av_s = average.GetAverage()
    del ecm_s, average
    out_av_s = "{0:s}{1:06d}_{2:04d}_s.srlz".format(InstCode, RunNumber,case_id)
    w = Manyo.WriteSerializationFileBinary(out_av_s)
    w.Save(ec_av_s)
    print("[o] Averaged histogram of south bank was serialized to {0}.".format(out_av_s))
    del w
    del ecm
print("Processed {0} sec.".format(time.time() - ini_time))



# -*- coding: utf-8 -*-
"""

"""
import vis.DetectMap as dp
import time
import vis.MPlot as mp
import vis.M2Plot as mp2
import vis.VisualContM as vctm

# 予約語の定義
ret = None 
vp1 = None
vp2 = None
vc = None
vc2 = None
vc3 = None
ec = None
earray = None
DAT = None
ECM = None
vcm = None
        
###################################################                            
def DetectMap(matrix=DAT):
        """
        Display Detector Map
        @param  matrix [Def:DAT] ElementContainerMatrix Object 
        @retval None
        """ 
        dp.MapFrame(None, matrix)    
        

###################################################                         
def MPlot(ec=ec):
        """
        Create 1D Plotter and Draw EC
        @param  ec [Def:EC]  ElementContainer
        @param  psd      PSD No.    
        @param  pixel    Pixel No.    
        @retval vp1    Instance of Plotter
        """ 
        # 親プロセス終了時に、プロッタを終了      
        return mp.MPlot(ec, True)
    
###################################################                            
def AddToMPlot(vp1=vp1, ec=ec):
        """
        Add EC to 1D Plotter
        @param  vp1 [Def:vp1]     Instance of 1D Plotter
        @param  ec [Def:EC]      ElementContainer
        @retval None
        """ 
        vp1.AddData(ec)  
        
###################################################                         
def M2Plot(matrix=ECM):
        """
        Create 2D Plotter and display 2D Map
        @param  matrix [Def:ECM]  ElementContainerArray
        @retval vp2 [Def:vp2]  Instance of 2D Plotter
        """ 
        # 親プロセス終了時に、プロッタを終了      
        return mp2.M2Plot(matrix, True)

###################################################
def ChangeMap(vp2=vp2, matrix=ECM):
        """
        Change 2D Map of M2Plot
	@param  vp2 [Def:vp2]     Instance of 2D Plotter
        @param  matrix [Def:ECM]  ElementContainerArray
        @retval None
        """ 
        # Map データの変更
        vp2.ChangeMap(matrix)	

###################################################                         
def VisCont(matrix=DAT):
        """
        Visualize Controller
        @param  matrix [Def:DAT] ElementContainerMatrix
        @retval vc   Instance of Controller
        """ 
        #       
        vc = vct.VisContFrame(matrix)
	return vc

###################################################                            
def AddToVisCont(vc=vc, matrix=DAT):
        """
        Add Matrix to Visual Controller
        @param  vc [Def:vc] Instance of Visual Controller
        @param  matrix [Def:DAT] ElementContainerMatrix
        @retval None
        """ 
        vc.AddMatrix(matrix)  

###################################################                         
def VisCont2(matrix=DAT):
        """
        Visualize Controller
        @param  matrix [Def:DAT] ElementContainerMatrix
        @retval vc2   Instance of Controller
        """ 
        #       
        vc2 = vct2.VisContFrame(matrix)
	return vc2

###################################################                            
def AddToVisCont2(vc2=vc2, matrix=DAT, tab=""):
        """
        Add Matrix to Visual Controller
        @param  vc2 [Def:vc2] Instance of Visual Controller
        @param  matrix [Def:DAT] ElementContainerMatrix
        @param  tab     Tab ID
        @retval None
        """ 
        dic_Tab = {"A":0,"B":1,"C":2,"D":3,"E":4,"F":5,"G":6,"H":7,"I":8,"J":9 }
        if tab!="":
		tab=tab.upper()
		if dic_Tab.has_key(tab.strip()):
			vc2.AddMatrix(matrix,dic_Tab[tab.strip()])  
		else:
			vc2.AddMatrix(matrix)
        else:
		vc2.AddMatrix(matrix)
###################################################                         
def VisContM(matrix=DAT):
        """
        Visualize Controller
        @param  matrix [Def:DAT]  ElementContainerMatrix
        @retval vcm   Instance of Controller
        """ 
        #       
        vcm = vctm.VisContMFrame(matrix)
	return vcm

###################################################                            
def AddToVisContM(vcm=vcm, matrix=DAT, tab=""):
        """
        Add Matrix to Visual Controller
        @param  vcm [Def:vcm]     Instance of Visual Controller
        @param  matrix [Def:DAT]  ElementContainerMatrix
        @param  tab     Tab ID
        @retval None
        """ 
        dic_Tab = {"A":0,"B":1,"C":2,"D":3,"E":4,"F":5,"G":6,"H":7,"I":8,"J":9 }
        if tab!="":
            tab = tab.upper()
            if dic_Tab.has_key(tab.strip()):
	        vcm.AddMatrix(matrix,dic_Tab[tab.strip()])  
            else:
		vcm.AddMatrix(matrix)
        else:
            vcm.AddMatrix(matrix)

           

# 関数名登録ディクショナリ
_functions = {"DetectMap":DetectMap,
              "MPlot":MPlot,
              "AddToMPlot":AddToMPlot,
              "M2Plot":M2Plot,
	      "ChangeMap":ChangeMap,
              "VisContM":VisContM,
              "AddToVisContM":AddToVisContM
	      }

_functionsOrder = [
	"DetectMap",
	"MPlot",
	"AddToMPlot",
	"M2Plot",
	"ChangeMap",
	"VisContM",
	"AddToVisContM"
	]
         



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function
import os
import sys
import time
import commands
"""
(2010.1.25) Bugfix for difference of directory name in /data/DAQ#/
"""


def _copydatasingle(runno=-1, flag=-1, target=""):
    """
    Copy or make symbolic link of Event Data from DAQ CPU to User CPU
    format Copydata(runno,[flag])
    @param runno(int) First Run No of a measurement to be copied
    @param flag(int)  Flag for Copy(0) or Symbolic link(1) / default is Symbolic link or -1 for testing
    @retval -1 No data in DAQ CPU
    @retval -2 False to make directory
    @retval -3 False to copy data file / make symbolic link
    @retval -4 Not agreement between origin data files and copied files
    @retval  0 success to execute
    """
    if flag == -1:
        print(" Copydata program runs on Dummy mode.")
    if runno == -1:
        print(" Copydata( first runno, last runno, flag=0:copy/1:Sybmolic link)")
        return
    RUNNO = "%06d" % runno
    # RUNNO = RUNNO.replace(" ", "0")
    HEAD = os.environ["UTSUSEMI_INST_CODE"] + RUNNO

    # ##Make List for pathes to EventData folder of given Run#
    if target == "":
        ROOTDIR_out = os.path.join(
            os.environ["UTSUSEMI_DATA_DIR"], os.environ["UTSUSEMI_INST_CODE"])
    else:
        ROOTDIR_out = target
    FOLDER_LIST = []  # for storing /data/DAQ#/XXX######_YYYYMMDD

    daq_id = 0
    FirstRunDir = ""
    while(True):
        DAQ_dir = "/data/DAQ%1d" % (daq_id)
        if not os.path.exists(DAQ_dir):
            break
        RunDir_list = os.listdir(DAQ_dir)
        RunDir = ""
        for A_dir in RunDir_list:
            if A_dir.find(HEAD) != -1:
                RunDir = A_dir
                if FirstRunDir == "":
                    FirstRunDir = A_dir
        if RunDir == "":
            print("Cannot find this Run Number :{} in {}".format(HEAD, DAQ_dir))
        else:
            FOLDER_LIST.append(os.path.join(DAQ_dir, RunDir))
        daq_id = daq_id + 1

    if len(FOLDER_LIST) == 0:
        print("No data of this Run Number:{} in all DAQ system.".format(HEAD))
        return -1

    '''
    ROOTDIRLIST=[]
    daq_id=0
    ROOTDIR="/data/DAQ%1d" % (daq_id)
    while(os.path.exists(ROOTDIR)):
        ROOTDIRLIST.append(ROOTDIR)
        daq_id=daq_id+1
        ROOTDIR="/data/DAQ%1d" % (daq_id)
    daq_no=daq_id-1

    ROOTDIR_out = os.environ["UTSUSEMI_DATA_DIR"]

    Folder_list = os.listdir(ROOTDIRLIST[0])
    Folder=""
    for A_Folder in Folder_list:
        if A_Folder.find(HEAD)!=-1:
            Folder=A_Folder
    if Folder=="":
        print "Cannot find such Run number =",HEAD
        return

    FOLDER_LIST=[]  ## for storing /data/DAQ#/XXX######_YYYYMMDD

    for FOLDER_d in ROOTDIRLIST:
        FOLDER_LIST.append( os.path.join(FOLDER_d,Folder) )

    FOLDER_to=os.path.join(ROOTDIR_out,Folder)  ## for storing /data/XXX/XXX######_YYYYMMDD
    '''
    # Make path and directory to output
    # for storing /data/XXX/XXX######_YYYYMMDD
    FOLDER_to = os.path.join(ROOTDIR_out, FirstRunDir)
    comm = "mkdir -p {}".format(FOLDER_to)
    print("Do command > {}".format(comm))
    if flag != -1:
        r1 = os.system(comm)
        if r1 != 0:
            print("False to make directory ({})".format(FOLDER_to))
            return -2

    # Make List of pathes for all event data to be copied
    EdataList = []
    for FOLDER_d in FOLDER_LIST:
        try:
            edata = os.listdir(FOLDER_d)

        except FileNotFoundError:
            print("Maybe there is no data folder. :{}".format(FOLDER_d))
        else:
            edata.sort()
            EdataList.append((FOLDER_d, edata))

    # Exucute coping files or making symbolic links of event data
    for (FOLDER, Edata) in EdataList:
        for onedata in Edata:
            if flag == 1:
                comm = "ln -s " + \
                    os.path.join(FOLDER, onedata) + " " + \
                    os.path.join(FOLDER_to, onedata)
            else:
                comm = "cp -fp " + \
                    os.path.join(FOLDER, onedata) + " " + \
                    os.path.join(FOLDER_to, onedata)
            print("Do command > {}".format(comm))
            if flag != -1:
                r1 = os.system(comm)
                if r1 != 0:
                    return -3

    # Check executed results
    if flag != -1:
        ES = []
        for F in FOLDER_LIST:
            try:
                E = os.listdir(F)
            except FileNotFoundError:
                print("Maybe there is no data folder :{}".format(F))
            else:
                ES = ES + E
        ES.sort()

        ET_tmp = os.listdir(FOLDER_to)
        ET = []
        for a_file in ET_tmp:
            if a_file[-4:] == ".edb" or a_file[-4:] == ".t0b":
                ET.append(a_file)
        ET.sort()

        for li in range(len(ES)):
            if ES[li] != ET[li]:
                print(
                    " WARNING!! : some files are not copied. {} -> {}".format(ES[li], ET[li]))

        if len(ES) != len(ET):
            if ET[-1] != FirstRunDir + "_info.txt":
                print(" ERROR!! : something was wrong!!.")
                return -4

    # Make information text about copied files
    Elist = []
    for FOLDER_d in FOLDER_LIST:
        ES = commands.getoutput("LANG='c';ls -l {}".format(FOLDER_d))
        EL = ES.split("\n")
        Elist = Elist + EL

    if flag != -1:
        fname = FirstRunDir + "_info.txt"
        ff = open(os.path.join(FOLDER_to, fname), "w")
        for line in Elist:
            ff.write(line + "\n")

        ff.close()

    else:
        print(Elist)

    return 0


def CopyEventData(firstrun=-1, lastrun=-1, flag=-1, target=""):
    """
    Copy Event data file

    @param firstrun (int) first run number
    @param lastrun  (int) last run number
    @param flag (int) 0:copy / 1:Symbolic link
    @retval list for return values of _copydatasingle
    """
    ret = []
    start_time = time.time()
    if (firstrun == -1) and (lastrun == -1):
        print(" Copydata( first runno, last runno, flag=0:copy/1:Sybmolic link)")
        return ret

    for runno in range(firstrun, (lastrun + 1)):
        rup_time = time.time()
        ret.append(_copydatasingle(runno, flag, target))
        print("time cost={}".format(time.time() - rup_time))
    print("Total time cost = {}".format(time.time() - start_time))

    return ret


if __name__ == '__main__':
    if len(sys.argv) == 4:
        firstrun = int(sys.argv[1])
        lastrun = int(sys.argv[2])
        flag = int(sys.argv[3])
        print("Copy data runno from {} to {}. Flag={}".format(
            firstrun, lastrun, flag))
        CopyEventdata(firstrun, lastrun, flag)
    elif len(sys.argv) == 5:
        firstrun = int(sys.argv[1])
        lastrun = int(sys.argv[2])
        flag = int(sys.argv[3])
        target = sys.argv[4]
        print("Copy data runno from {} to. Flag={}".format(
            firstrun, lastrun, flag))
        CopyEventdata(firstrun, lastrun, flag, target)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os


def CheckForOutput( parent_dir ):
    """
    出力用パスをチェックする
    1. ~/ana/DNA/<parent_dir> ($UTSUSEMI_USR_PRIV_HOME/ana/DNA/<parent_dir>/)
    2. Current Path + <parent_dir>/
    
    @param parent_dir (str) "Absorb", "database", "Efficiency", "EOrg", "Mask", "setup" など
    @return full_path_dir : 最終的に使用するフォルダのパスを返す
    @retrun ""            : 見つからなかった場合
    """

    # Check ~/ana/DNA/<parent_dir>
    if "UTSUSEMI_INST_CODE" in os.environ:
        inst_code = os.environ["UTSUSEMI_INST_CODE"]
    elif "UTSUSEMI_SYS_NAME" in os.environ:
        inst_code = os.environ["UTSUSEMI_SYS_NAME"]
    else:
        inst_code = "DNA"

    if "UTSUSEMI_USR_PRIV_HOME" in os.environ:
        user_dir = os.environ["UTSUSEMI_USR_PRIV_HOME"]
    elif "UTSUSEMI_USR_DIR" in os.environ:
        user_dir = os.environ["UTSUSEMI_USR_DIR"]

    full_path_dir = os.path.join( user_dir, "ana", inst_code, parent_dir )
    if os.path.isdir( full_path_dir ):
        if os.access( full_path_dir, os.W_OK ):
            return full_path_dir
        else:
            print("SearchPathDNA::CheckForOutput >> "+" cannot write / make new file in "+full_path_dir)

    # Check Current Path + <parent_dir>
    full_path_dir = os.path.join( os.getcwd(), parent_dir )
    if os.path.isdir( full_path_dir ):
        if os.access( full_path_dir, os.W_OK ):
            return full_path_dir
        else:
            print("SearchPathDNA::CheckForOutput >> "+" cannot write / make new file in "+full_path_dir)
    else:
        try:
            os.makedirs( full_path_dir )
        except:
            print("SearchPathDNA::CheckForOutput >> "+" cannot make new directry as "+full_path_dir)
            return ""

    return full_path_dir

def CheckForInput( parent_dir, file_name ):
    """
    入力用パスとファイルをチェックする
    1. ~/ana/DNA/<parent_dir>/<file_name> ($UTSUSEMI_USR_PRIV_HOME/ana/DNA/<parent_dir>/<file_name>)
    2. Current Path + <parent_dir>/<file_name>
    
    @param parent_dir (str) "Absorb", "database", "Efficiency", "EOrg", "Mask", "setup" など
    @param file_name (str)
    @return "" : ファイルが見つからない
    @return /full/path/to/file : ファイルが見つかった
    """
    # Check ~/ana/DNA/
    if "UTSUSEMI_INST_CODE" in os.environ:
        inst_code = os.environ["UTSUSEMI_INST_CODE"]
    elif "UTSUSEMI_SYS_NAME" in os.environ:
        inst_code = os.environ["UTSUSEMI_SYS_NAME"]
    else:
        inst_code = "DNA"

    if "UTSUSEMI_USR_PRIV_HOME" in os.environ:
        user_dir = os.environ["UTSUSEMI_USR_PRIV_HOME"]
    elif "UTSUSEMI_USR_DIR" in os.environ:
        user_dir = os.environ["UTSUSEMI_USR_DIR"]

    full_path_dir = os.path.join( user_dir, "ana", inst_code, parent_dir )
    if os.path.isdir( full_path_dir ):
        full_path_file = os.path.join( full_path_dir, file_name )
        if os.path.exists( full_path_file ):
            return full_path_file

    # Check CurrentPath/<parent_dir>/<file_name>
    full_path_dir = os.path.join( os.getcwd(), parent_dir )
    if os.path.isdir( full_path_dir ):
        full_path_file = os.path.join( full_path_dir, file_name )
        if os.path.exists( full_path_file ):
            return full_path_file

    return ""

if __name__ == '__main__':
    targets = ["Absorb", "database", "Efficiency", "EOrg", "Mask", "setup"]
    for a_folder in targets:
        ret=CheckForOutput( a_folder )
        if ret=="":
            print( "Failed to CheckForOutput return value=%s."%(a_folder))
        else:
            print( "Succeeded to CheckForOutput to %s"%(ret))

    for a_folder in targets:
        ret = CheckForInput( a_folder, "test.txt" )
        if ret=="":
            print( "Failed to CheckForIntput in %s."%(a_folder))
        else:
            print( "Succeeded to CheckForInput to %s"%(ret))



#!/usr/bin/python

from __future__ import print_function
import math
import os
import glob
import time

start = time.time()
import sys
ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
else:
    from xml.etree.ElementTree import ElementTree

def _writeParam(val,dpath,params,att="",i=0):
    if dpath!="":
        headStr=""
        head=open(dpath)
        l=head.readline()
        if l[:2]=="<?":headStr+=l
        l=head.readline()
        if l[:2]=="<!":headStr+=l

        fd = open(dpath,'rb')
        dom=ElementTree(file=fd)
        fd.close()

        ele = dom.getroot()
        for param in params:
            if param==params[-1]:
                ele = ele.findall(param)
            else:
                ele = ele.find(param)
        valStr=str(val)

        if att=="":
            ele[i].text=valStr
        else:
            ele[i].set(att,valStr)
        dom.write(dpath)

        allline=open(dpath,"r").read()

        f = open(dpath,"w")
        f.write(headStr)
        for line in allline:f.write(line)
        f.close()

def _readParam(dpath,params,att="",i=0):
    if dpath!="":
        fd = open(dpath,'rb')
        dom=ElementTree(file=fd)
        fd.close()

        ele = dom.getroot()
        for param in params:
            if param==params[-1]:
                ele = ele.findall(param)
            else:
                ele = ele.find(param)

        if att=="":
            return ele[i].text
        else:
            return ele[i].get(att)


def _writeBinInfo(filename,initial,end,width,separator):
    f = open(filename,"w")
    i=0
    while 1:
        val =  initial + i*width
        if val >end:
            break
        f.write(str(val) + separator)
        i+=1
    f.close()

def _readBinInfo(filename,separatorIn,separatorOut):
    f = open(filename,"r")
    lines = f.read().split(separatorIn)

    lines = [line for line in lines if line !=""]

    for i,line in enumerate(lines):
        print("  "+str(i)+"\t"+line)

    text  = separatorOut.join(lines)
    f.close()
    return text

#initial  = -1.0
#end      =  1.0
#width    =  0.1
#separator =  "\n"
#filename = "binInfo2.txt"
#_writeBinInfo(filename,initial,end,width,separator)

#separatorIn  =  "\n"
#separatorOut =  ","

#ret=_readBinInfo(filename,separatorIn,separatorOut)

#print ret



#! /usr/bin/env python
# -*- coding:utf-8 -*-
import tkinter
from tkinter import filedialog as tkFileDialog
import datetime
import time
import os
import Cmm
import uGao.MPlot as MP
import uGao.M2PlotPlusCui as MP2
import utsusemi.DNA.MultiDataCombine.subUtils.BinInfoReaderBase as MDCR
import EneOrig
import subprocess
import tkinter.font as tkfont

class App(tkinter.Frame):
    def __init__(self, master = None):
    # tkinter.Frame.__init__(self, master)
        if ( FastChopperUse == True):
            tkinter.Frame.__init__(self, master, background="#2DE56B")
            self.master.title("DNA SQW Reduction With Fast Chopper")
            self.errorstr = tkinter.StringVar()
            self.errorstr.set("High Resolution Mode.")
        else:
            tkinter.Frame.__init__(self, master)
            self.master.title("DNA SQW Reduction Without Fast Chopper")
            self.errorstr = tkinter.StringVar()
            self.errorstr.set("Low Resolution Mode.")

        # Si(111) self.si == 0, Si(333) self.si == 1
        self.si = 0
        self.v = tkinter.IntVar()

        display = tkinter.Label(self, textvariable = self.errorstr, font = ('Helvetica', '16', 'bold'))
        display.pack( side = tkinter.TOP)
        f0 = tkinter.LabelFrame(self, text = 'Si')
        f01 = tkinter.LabelFrame(f0, relief = tkinter.FLAT)
        self.v.set(0)
        rd = tkinter.Radiobutton(f01, text = 'Si(111) : dQ=', value = 0, variable = self.v, command = self.sel_si)
        rd.pack(padx=2, pady=2)
        rd = tkinter.Radiobutton(f01, text = 'Si(333) : dQ=', value = 1, variable = self.v, command = self.sel_si)
        rd.pack(padx=2, pady=2)
        rd = tkinter.Radiobutton(f01, text = 'Si(311) : dQ=', value = 2, variable = self.v, command = self.sel_si)
        rd.pack(padx=2, pady=2)
        f01.pack(padx=0,pady=2, side = tkinter.LEFT)
        f02 = tkinter.LabelFrame( f0, relief = tkinter.FLAT)

        f021 = tkinter.LabelFrame( f02, relief = tkinter.FLAT)
        self.dqs1 = tkinter.Spinbox( f021, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        self.dqs1.delete( 0, tkinter.END)
        self.dqs1.insert( 0, '0.05')
        self.dqs1.pack(padx=0,pady=0, side = tkinter.LEFT)
        rd = tkinter.Label(f021, text = " Qmin=0.0 Qmax=2.0")
        rd.pack(padx=0,pady=0, side = tkinter.LEFT)
        f021.pack(padx=0,pady=2, fill = tkinter.BOTH)

        f022 = tkinter.LabelFrame(f02, relief = tkinter.FLAT)
        self.dqs3 = tkinter.Spinbox( f022, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        self.dqs3.delete( 0, tkinter.END)
        self.dqs3.insert( 0, '0.10')
        self.dqs3.pack(padx=0,pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f022, text = " Qmin=0.0 Qmax=6.0")
        rd.pack(padx=0,pady=0, side = tkinter.LEFT)
        f022.pack(padx=0,pady=2, fill = tkinter.BOTH)

        f023 = tkinter.LabelFrame(f02, relief = tkinter.FLAT)
        self.dqs4 = tkinter.Spinbox( f023, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        self.dqs4.delete( 0, tkinter.END)
        self.dqs4.insert( 0, '0.10')
        self.dqs4.pack(padx=0,pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f023, text = " Qmin=0.0 Qmax=4.0")
        rd.pack(padx=0,pady=0, side = tkinter.LEFT)
        f023.pack(padx=0,pady=2, fill = tkinter.BOTH)

        f02.pack(padx=0, pady=2, side = tkinter.LEFT)
        f0.pack(padx=5, pady=5)

        f1 = tkinter.LabelFrame(self, text = 'Run No.')
        rd = tkinter.Label(f1, text = "       ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.run = tkinter.Spinbox( f1, from_=1000, to=50000, increment=1, width=5)
        self.run.delete( 0, tkinter.END)
        self.run.insert( 0, '2000')
        self.run.pack(pady=2,side = tkinter.LEFT)
        rd = tkinter.Label( f1, text = "    Temp. Sensor")
        rd.pack(pady=0, side = tkinter.LEFT)
#.T.Yamada start
        ChList = ("A","B","C","C2","D","D2","D3","D4","D5","C_TC","D_TC")
        self.Ch = tkinter.StringVar()
        self.Ch.set(ChList[6])
        self.ChMenu = tkinter.OptionMenu( f1, self.Ch, *ChList)
        self.ChMenu.pack(pady=2, side = tkinter.LEFT)
#T.Yamada End
        button2 = tkinter.Button( f1, text = "Read Temp.", command = self.readtemp)
        button2.pack( side = tkinter.LEFT)	
        f1.pack(padx=5, pady=5, fill = tkinter.BOTH)
        f2 = tkinter.LabelFrame(self, text = 'Parameter')
        f20 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        rd = tkinter.Label( f20, text="λcenter ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.lam = tkinter.Spinbox( f20, from_=0, to=99.999, increment=0.001, width=6, format='%6.3f')
        self.lam.delete( 0, tkinter.END)
# 
        if ( FastChopperUse == True):
            self.lam.insert( 0, ' 6.321')
        else:
            self.lam.insert( 0, ' 6.321')

        self.lam.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f20, text="[A]/ dλ ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.dl = tkinter.Spinbox( f20, from_=0, to=9.999, increment=0.001, width=5, format='%5.3f')
        self.dl.delete( 0, tkinter.END)
        self.dl.insert( 0, '4.15')
        self.dl.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f20, text="[A]")
        rd.pack(pady=0, side = tkinter.LEFT)
        f20.pack(padx=5, pady=2, fill = tkinter.BOTH)

        f21 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        rd = tkinter.Label( f21, text="Emin ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.emin = tkinter.Spinbox( f21, from_=-99, to=999, increment=1, width=4, format='%3.0f')
        self.emin.delete( 0, tkinter.END)
# 
        if ( FastChopperUse == True):
            self.emin.insert( 0, '-40')
        else:
            self.emin.insert( 0, '-500')

        self.emin.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f21, text="μeV/ Emax ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.emax = tkinter.Spinbox( f21, from_=0, to=999, increment=1, width=4, format='%3.0f')
        self.emax.delete( 0, tkinter.END)
# 
        if ( FastChopperUse == True):
            self.emax.insert( 0, '100')
        else:
            self.emax.insert( 0, '1500')

        self.emax.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f21, text="μeV/ dE ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.de = tkinter.Spinbox( f21, from_=0.01, to=99, increment=0.01, width=5, format='%5.2f')
        self.de.delete( 0, tkinter.END)
# 
        if ( FastChopperUse == True):
            self.de.insert( 0, ' 0.50')
        else:
            self.de.insert( 0, ' 2')

        self.de.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label(f21, text="μeV")
        rd.pack(pady=0, side = tkinter.LEFT)
        f21.pack(padx=5, pady=2, fill = tkinter.BOTH)
#T. Yamada Add[20150601]
        self.Ebin_flag = tkinter.BooleanVar()
        self.EbinFile_name = "not selected"
        
        #f3=tkinter.LabelFrame(self, text='Detector Efficiency Correction')
        f21_1 = tkinter.LabelFrame(f2, relief = tkinter.FLAT)
        cb = tkinter.Checkbutton(f21_1, text='User difined E Bin',variable = self.Ebin_flag).pack(padx=2, pady=2, side = tkinter.LEFT)
        button5 = tkinter.Button(f21_1, text="Select file", command = self.loadEbinfile).pack(padx=2, pady=2, side = tkinter.LEFT)
        
        #f21_2 = tkinter.Frame(f2,relief=tkinter.FLAT)
        self.cb21_3 = tkinter.Label(f21_1, text = self.EbinFile_name)
        self.cb21_3.pack(padx = 20, pady=2,side = tkinter.LEFT)
        #f21_2.pack(padx=0, pady=2, fill = tkinter.BOTH)
        f21_1.pack(padx=0, pady=2, fill = tkinter.BOTH)
#T. Yamada End [20150601]

        f22 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        rd = tkinter.Label( f22, text = "T0Shift ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.T0S = tkinter.Spinbox( f22, from_=0, to=99999, increment=1, width=5, format='%5.0f')
        self.T0S.delete( 0, tkinter.END)
# 
        if ( FastChopperUse == True):
            self.T0S.insert( 0, '12325')
        else:
            self.T0S.insert( 0, '120')

        self.T0S.pack(pady=2, side = tkinter.LEFT)
        rd = tkinter.Label( f22, text="[μs] ")
        rd.pack(pady=0, side = tkinter.LEFT)
        f22.pack(padx=5, pady=2, fill = tkinter.BOTH)

        f23 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        rd = tkinter.Label( f23, text = "StartTime")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.Sta = tkinter.Spinbox( f23, from_=0, to=9999999, increment=1, width=7, format='%7.0f')
        self.Sta.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f23, text = "[sec.]/ EndTime ")
        rd.pack(pady=0, side = tkinter.LEFT)
        self.End = tkinter.Spinbox( f23,from_=0,to=9999999,increment=1,width=7,format='%7.0f')
        self.End.delete( 0, tkinter.END)
        self.End.insert( 0, '9999999')
        self.End.pack(pady=0, side = tkinter.LEFT)
        rd = tkinter.Label( f23, text = "[sec.]")
        rd.pack(pady=0, side = tkinter.LEFT)
        f23.pack(padx=5,pady=2, fill = tkinter.BOTH)
        f2.pack(padx=5, pady=5, fill = tkinter.BOTH)

#T. Yamada Add[20140524]
        self.DetEffCr_flag = tkinter.BooleanVar()
        self.DetEffFile_name = "not selected"
        
        f3=tkinter.LabelFrame(self, text='Detector Efficiency Correction')
        f31 = tkinter.LabelFrame(f3, relief = tkinter.FLAT)
        cb = tkinter.Checkbutton(f31, text='Vanadium Correction',variable = self.DetEffCr_flag).pack(padx=2, pady=2, side = tkinter.LEFT)
        button4 = tkinter.Button(f31, text="Select file", command = self.loadfile).pack(padx=2, pady=2, side = tkinter.LEFT)
        f31.pack(padx=0, pady=2, fill = tkinter.BOTH)
        #f32 = tkinter.Frame(f3,relief=tkinter.FLAT)
        self.cb31 = tkinter.Label(f31, text = self.DetEffFile_name)
        self.cb31.pack(padx = 20, pady=2,side = tkinter.LEFT)
        #f32.pack(padx=0, pady=2, side = tkinter.LEFT)
        f3.pack(padx=5, pady=5, fill = tkinter.BOTH)
#T. Yamada End [20140524]
#T. Yamada Add [20141110]
        #self.DoMask_flag = tkinter.BooleanVar()
        self.MaskFile_name = "maskTY.txt"
        
        f4=tkinter.LabelFrame(self, text='Select mask file')
        f41 = tkinter.LabelFrame(f4, relief = tkinter.FLAT)
        #cb = tkinter.Checkbutton(f41, text='Vanadium Correction',variable = self.DetEffCr_flag).pack(padx=2, pady=2, side = tkinter.LEFT)
        button4 = tkinter.Button(f41, text="Select file", command = self.loadMaskfile).pack(padx=2, pady=2, side = tkinter.LEFT)
        #f41.pack(padx=0, pady=2, fill = tkinter.BOTH)
        #f42 = tkinter.Frame(f4,relief=tkinter.FLAT)
        self.cb41 = tkinter.Label(f41, text = self.MaskFile_name)
        self.cb41.pack(padx = 20, pady=2,side = tkinter.LEFT)
        f41.pack(padx=0, pady=2, fill = tkinter.BOTH)
        #f42.pack(padx=0, pady=2, side = tkinter.LEFT)
        f4.pack(padx=5, pady=5, fill = tkinter.BOTH)
#T. Yamada End [20141110]

        button1 = tkinter.Button(self, text = "Launch Data Reduction", command = self.launch)
        button3 = tkinter.Button(self, text = "Quit", command = self.master.destroy)
        button1.pack(side = tkinter.LEFT)
        button3.pack(side = tkinter.RIGHT)
        self.pack()
# self.step == 10: launch stand-by
        self.step = 11

    def loadfile(self):
        def _getcorrefftype(fname):
            f = open(fname,"r")
            line = f.readline()
            f.close()
            ret = 0
            if line.find(",") != -1:
                ret = len(line.split(",")) #ret = 120
            else:
                ret = len(line.split(" ")) #ret = 121
            
            if ret != 120 and ret !=121:
                print ("File is incorrect. Check the file {0}, {1}".format(fname, ret))

            return ret
        
        self.path_name = "/home/dnaadmin/ana/DNA"
        self.DetEffFile_name = tkFileDialog.askopenfilename(initialdir = self.path_name)
        self.cb31.configure(text=self.DetEffFile_name)
        self.DetEffFileType = _getcorrefftype(self.DetEffFile_name)
        
        print (self.DetEffFile_name, self.DetEffCr_flag.get(),self.DetEffFileType)
    

    def loadMaskfile(self):
        self.path_name = "/home/dnaadmin/ana/DNA/Mask"
        self.MaskFile_name = tkFileDialog.askopenfilename(initialdir = self.path_name,filetypes=[('mask file','.txt')])
        self.MaskFile_name = os.path.basename(self.MaskFile_name)
        self.cb41.configure(text=self.MaskFile_name)
        print (self.MaskFile_name)
       
    def loadEbinfile(self):
        self.path_name = "/home/dnaadmin/"
        self.EbinFile_name = tkFileDialog.askopenfilename(initialdir = self.path_name)
        self.cb21_3.configure(text=self.EbinFile_name)

    def sel_si(self):
        if self.step == 11:
            self.si = self.v.get()
            if self.si != 2:
                self.MaskFile_name = "maskTY.txt"
                self.cb41.configure(text=self.MaskFile_name)
            elif self.si == 2:
                self.MaskFile_name = "mask311.txt"
                self.cb41.configure(text=self.MaskFile_name)

    def readtemp(self):
        if self.step == 11:
            self.runNo = int(self.run.get())
            TCh = self.Ch.get()
            print ("RunNo:{0:d}, TempCh: {1}".format(self.runNo,TCh))
            #get Run No.
            print ("SQWGUI--> Reading Temperature start")
            EC = Cmm.CheckSampleTempByRunNumber( self.runNo,TCh)
            print ("SQWGUI--> Reading Temperature end")
            MP.MPlot(EC)
            self.errorstr.set("Temperature run# %d Ch %s" % (self.runNo, TCh))

    def launch(self):
        print ("koko283", self.step)
        if self.step == 11:
            self.mydatetime = datetime.datetime.now()
            datetimestr = self.mydatetime.strftime('%Y/%m/%d %H:%M:%S')
            datestr = self.mydatetime.strftime('Utsusemi%Y%m%d.txt')
            #Get Run No.
            self.runNo = int(self.run.get())

            #Get Q parameter
            self.Qmin = 0.0
            if self.si == 0:#Si(111)
                self.DQ = float(self.dqs1.get())
                self.Qmax = 2.0
            elif self.si == 1:#Si(333)
                self.DQ = float(self.dqs3.get())
                self.Qmax = 6.0
            elif self.si == 2:
                self.DQ = float(self.dqs4.get())
                self.Qmax = 4.0
            
            QParam = "%.1f,%.3f,%.3f" %(self.Qmin,self.Qmax,self.DQ)	
            print ("->>QParam: {0}".format(QParam))

            #Get Lambda Parameter
            # LambdaCenter = float(self.lam.get())
            # dLambda = float(self.dl.get())
            self.LambdaParam = "%.4f,%.4f" %(float(self.lam.get()),float(self.dl.get()))
            print ("->>LambdaParam: {0}".format(self.LambdaParam))

            #Get HWParam
            # Emin = float(self.emin.get())/1000.0
            # Emax = float(self.emax.get())/1000.0
            # dE = float(self.de.get())/1000.0
            self.HWParam = "%.5f,%.5f,%.5f" %((float(self.de.get())/1000.0),(float(self.emin.get())/1000.0),(float(self.emax.get())/1000.0))
            print ("->>HWParam: {0}".format(self.HWParam))

            #Get T0Shift
            self.T0Shift = float(self.T0S.get())
            print ("->>T0Shift: {0}".format( self.T0Shift))

            #Get Time Parameter
            # startTime = int(self.Sta.get())
            # endTime = int(self.End.get())
            self.TimeParam = "%d,%d" %(int(self.Sta.get()), int(self.End.get()))
            print ("->>TimeParam: {0}".format(self.TimeParam))

            #Write log file
            self.fp = open( "%s" % (datestr), "a")
            self.fp.write( "%s run:%d Si" % (datetimestr, self.runNo))
            if self.si == 0:
                self.fp.write( "(111):de:%s min:%3.1f max:%3.1f" % (self.dqs1.get(),0,2.0))
            elif self.si == 1:
                self.fp.write( "(333):de:%s min:%3.1f max:%3.1f" % (self.dqs3.get(),0,6.0))
            elif self.si == 2:
                self.fp.write( "(311):de:%s min:%3.1f max:%3.1f" % (self.dqs4.get(),0,6.0))
            self.fp.write(" lambda:%s dlambda:%s" % (self.lam.get(), self.dl.get()))
            self.fp.write(" emin:%s emax:%s de:%s" % (self.emin.get(), self.emax.get(),self.de.get()))
            self.fp.write(" T0shift:%s" % (self.T0S.get()))
            self.fp.write(" Start:%s End:%s" % (self.Sta.get(), self.End.get()))
            self.fp.write(" Maskfile:%s" % (self.MaskFile_name))
            self.fp.write(" EfficiencyFile:%s %s\n" % ( self.DetEffFile_name, self.DetEffCr_flag.get()))
            self.fp.close()

            self.step = 0
            self.errorstr.set("Check Data  (0/11)")
            self.after( 100, self.countup)

    def countup(self):
        if self.step == 0:
            copyresult, messg = CopyEventData(int(self.runNo))
            self.errorstr.set(messg)
            if copyresult == 1:
                self.step = 10
            else:
                self.errorstr.set("GetMonHistogram.  (1/11)")
                self.EC = Cmm.GetHistogramMon(self.runNo, True, self.LambdaParam, self.T0Shift, 0.0, False, self.TimeParam, FastChopperUse)  
                self.errorstr.set("MultiplyConstant. (2/11)")
                self.step = 1
        elif self.step == 1:
            Cmm.MutiplyConstant(self.EC, 1e-09)
        
            if self.si == 0:
                self.errorstr.set("GetHistogramHW.   (3/11)")
            elif self.si == 1:
                self.errorstr.set("GetHistogramHW333.(3/11)")
            elif self.si == 2:
                self.errorstr.set("GetHistogramHW311.(3/11)")
            self.step = 2
        
        elif self.step == 2:
            if self.Ebin_flag.get():
                self.HWParam = MDCR._readBinInfo(self.EbinFile_name,"\n",",")
                #print "koko", self.HWParam
                #print "kokoko"
                #return -1

            if self.si == 0:
                self.DAT = Cmm.GetHistogramHW( self.runNo, self.HWParam, self.LambdaParam, self.T0Shift, False, self.TimeParam, FastChopperUse)
            elif self.si == 1:
                self.DAT = Cmm.GetHistogramHW333( self.runNo, self.HWParam, self.LambdaParam, self.T0Shift, False, self.TimeParam, FastChopperUse)
            elif self.si == 2:
                self.DAT = Cmm.GetHistogramHW311( self.runNo, self.HWParam, self.LambdaParam, self.T0Shift, False, self.TimeParam, FastChopperUse)
            self.errorstr.set("DoMask.           (4/11)")
            self.step = 3
        elif self.step == 3:
            """
            [20141110 Yamada Modified]
            if self.si != 2:
                Cmm.DoMask(self.DAT, "maskTY.txt")
            else:
                Cmm.DoMask(self.DAT, "mask311.txt")
            """
            Cmm.DoMask(self.DAT, self.MaskFile_name)
            self.errorstr.set("ILambdaCorrDNA.   (5/11)")
            self.step = 4
        elif self.step == 4:
            self.ECM = Cmm.ILambdaCorrDNA( self.DAT, self.EC, True)
            self.errorstr.set("SolidAngleCorrDNA.(6/11)")
            self.step = 5
        elif self.step == 5:
            if self.DetEffCr_flag.get() and self.DetEffFileType == 120:
                self.ECM2 = Cmm.SolidAngleCorrDNA( self.ECM, True, False, False, "sample", DetEffDataPath=self.DetEffFile_name )
                self.ECM2 = EneOrig.DoEfficiencyCorrection(dat=self.ECM2,filename=self.DetEffFile_name,mode=0)
            else:
                self.ECM2 = Cmm.SolidAngleCorrDNA( self.ECM, True, False, self.DetEffCr_flag.get(), "sample", DetEffDataPath=self.DetEffFile_name )
            self.errorstr.set("MultiplyConstant. (7/11)")
            self.step = 6
        elif self.step == 6:
            Cmm.MutiplyConstant( self.ECM2, 1e-06)
            self.errorstr.set("CreateQEMap.      (8/11)")
            self.step = 7
        elif self.step == 7:
            Cmm.KiKfCorrection(self.ECM2)
            self.errorstr.set("KiKfCorrection.      (9/11)")
            self.step = 8
        elif self.step == 8:
            self.ECA = Cmm.CreateQEMap( self.ECM2, self.Qmin, self.Qmax, self.DQ)
            self.errorstr.set("OutoutECA.        (10/11)")
            self.step = 9
        elif self.step == 9:
            footer = "_GUIQE"
            if self.DetEffCr_flag.get():
                footer = "_VGUIQE"
            Cmm.OutputECA( self.ECA, footer)
            Cmm.OutputECAText( self.ECA, footer)
            Cmm.OutputECADAVEText( self.ECA, footer)
            #T.Yamada Add[20150601]
            if self.Ebin_flag.get():
                Cmm.OutputNeiECAText(self.ECA,"_neimMDRQE")
            #T.Yamada End[20150601]
            
            self.errorstr.set("M2PLOT(ECA)      (11/11)")
            self.step = 10
        elif self.step == 10:
            MP2.M2PlotPlusCui(self.ECA).ShowData()
            self.errorstr.set("M2PLOT end. run# %d" % (self.runNo))
            self.step = 11
        if self.step < 11:
            self.after(200, self.countup)

def CopyEventData(RunNo):
    #if RunNo < 14082:
    if RunNo < 0:
        command = "ssh dnaadmin@192.168.1.10 /home/dnaadmin/Yamada/01_CopyEventData/CopyEventEdata.sh %d" %(RunNo)
        args = command.split(" ")
        proc = subprocess.Popen(args, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        stdout_data, stderr_data = proc.communicate()
        print ("stdout_data >> {0}".format(stdout_data.decode('utf-8')))
        print ("stderr_data >> {0}".format(stderr_data.decode('utf-8').replace("\n","")))
        result = proc.returncode
        #proc.close()
        if result == 1:
            messg =  "DNA%06d_* is not found on /data and /ifs_data/DNA/DNA" %RunNo
        else:
            messg =  "DNA%06d_* is ready for the reduction." %RunNo
    else:
        result = 0
        messg =  "DNA%06d_* is ready for the reduction." %RunNo

    return result, messg

if __name__ == "__main__":

	if ( os.environ.get("FastChopperUse")=="True"):
	    FastChopperUse = True
	else:
	    FastChopperUse = False
	app = App()
	app.mainloop()



#! /usr/bin/env python
# -*- coding:utf-8 -*-
import datetime
import time
import os
import sys
import Cmm
import uGao.MPlot as MP
import uGao.M2PlotPlusCui as MP2
import utsusemi.DNA.MultiDataCombine.subUtils.BinInfoReaderBase as MDCR
import EneOrig
import subprocess
import Manyo.Utsusemi as MU

PYSIDEVER = 6
try:
    from PySide6 import QtCore, QtGui, QtWidgets
    from PySide6.QtWidgets import QApplication, QFileDialog, QListWidgetItem, QMessageBox
    from PySide6.QtCore import QFile
    from ui6_SQW import Ui_MainWindow
except:
    from PySide2 import QtCore, QtGui, QtWidgets
    # from PySide2.QtUiTools import QUiLoader
    from PySide2.QtWidgets import QApplication, QFileDialog, QListWidgetItem, QMessageBox
    from PySide2.QtCore import QFile
    from ui2_SQW import Ui_MainWindow
    PYSIDEVER = 2


class App(QtWidgets.QMainWindow):
    def __init__(self, master = None):
        super(App, self).__init__(master)
        self.frame = Ui_MainWindow()
        self.frame.setupUi(self)

        self.errorstr = self.frame.errorstr
        
        """
        self.lam =
        self.dl =
        self.emin =
        self.emax =
        self.de =
        self.cb_UserDefinedEBin =  # => self.Ebin_flag
        self.bt_UserDefinedEBin =  # button5
        self.cb21_3 = # "txt_EbinFileName" <- self.cb21_3  => self.EbinFile_name
        self.T0S =
        self.Sta =
        self.End =
        self.cb_VanadiumCorr =  # => self.DetEffCr_flag
        self.bt_VanadiumCorr = # button4
        self.txt_VanadiumCorr = # => self.DetEffFile_name
        self.bt_MaskFile = 
        #self.txt_MaskFile = 
        self.cb41 = self.frame.txt_MaskFile # => self.MaskFile_name = "maskTY.txt"
        """
        
        
        if ( FastChopperUse == True):
            #self.setStyleSheet("background-color: lime")
            self.frame.background.setStyleSheet("background-color: lime")
            #tkinter.Frame.__init__(self, master, background="#2DE56B")
            #self.master.title("DNA SQW Reduction With Fast Chopper")
            self.setWindowTitle("DNA SQW Reduction With Fast Chopper")
            #self.errorstr = tkinter.StringVar()
            #self.errorstr.set("High Resolution Mode.")
            self.errorstr.setText("High Resolution Mode.")
        else:
            #tkinter.Frame.__init__(self, master)
            #self.master.title("DNA SQW Reduction Without Fast Chopper")
            self.setWindowTitle("DNA SQW Reduction Without Fast Chopper")
            #self.errorstr = tkinter.StringVar()
            #self.errorstr.set("Low Resolution Mode.")
            self.errorstr.setText("Low Resolution Mode.")

        # Si(111) self.si == 0, Si(333) self.si == 1
        self.si = 0
        #self.v = tkinter.IntVar()

        #display = tkinter.Label(self, textvariable = self.errorstr, font = ('Helvetica', '16', 'bold'))
        #display.pack( side = tkinter.TOP)
        #f0 = tkinter.LabelFrame(self, text = 'Si')
        #f01 = tkinter.LabelFrame(f0, relief = tkinter.FLAT)
        #self.v.set(0)
        #rd = tkinter.Radiobutton(f01, text = 'Si(111) : dQ=', value = 0, variable = self.v, command = self.sel_si)
        #rd.pack(padx=2, pady=2)
        #rd = tkinter.Radiobutton(f01, text = 'Si(333) : dQ=', value = 1, variable = self.v, command = self.sel_si)
        #rd.pack(padx=2, pady=2)
        #rd = tkinter.Radiobutton(f01, text = 'Si(311) : dQ=', value = 2, variable = self.v, command = self.sel_si)
        #rd.pack(padx=2, pady=2)
        #f01.pack(padx=0,pady=2, side = tkinter.LEFT)
        #f02 = tkinter.LabelFrame( f0, relief = tkinter.FLAT)

        self.frame.rb_Si111.clicked.connect(self.sel_si)
        self.frame.rb_Si333.clicked.connect(self.sel_si)
        self.frame.rb_Si311.clicked.connect(self.sel_si)
        self.rb_group = QtWidgets.QButtonGroup()
        self.rb_group.addButton(self.frame.rb_Si111, 1)
        self.rb_group.addButton(self.frame.rb_Si333, 2)
        self.rb_group.addButton(self.frame.rb_Si311, 3)
        self.v = 0
        self.rb_group.button(self.v + 1).setChecked(True)

        #f021 = tkinter.LabelFrame( f02, relief = tkinter.FLAT)
        #self.dqs1 = tkinter.Spinbox( f021, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        #self.dqs1.delete( 0, tkinter.END)
        #self.dqs1.insert( 0, '0.05')
        #self.dqs1.pack(padx=0,pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label(f021, text = " Qmin=0.0 Qmax=2.0")
        #rd.pack(padx=0,pady=0, side = tkinter.LEFT)
        #f021.pack(padx=0,pady=2, fill = tkinter.BOTH)
        self.dqs1 = self.frame.dsp_Si111
        self.dqs1.setMaximum(9.99)
        self.dqs1.setMinimum(0.01)
        self.dqs1.setSingleStep(0.01)
        self.dqs1.setValue(0.05)

        #f022 = tkinter.LabelFrame(f02, relief = tkinter.FLAT)
        #self.dqs3 = tkinter.Spinbox( f022, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        #self.dqs3.delete( 0, tkinter.END)
        #self.dqs3.insert( 0, '0.10')
        #self.dqs3.pack(padx=0,pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label( f022, text = " Qmin=0.0 Qmax=6.0")
        #rd.pack(padx=0,pady=0, side = tkinter.LEFT)
        #f022.pack(padx=0,pady=2, fill = tkinter.BOTH)
        self.dqs3 = self.frame.dsp_Si333
        self.dqs3.setMaximum(9.99)
        self.dqs3.setMinimum(0.01)
        self.dqs3.setSingleStep(0.01)
        self.dqs3.setValue(0.05)

        #f023 = tkinter.LabelFrame(f02, relief = tkinter.FLAT)
        #self.dqs4 = tkinter.Spinbox( f023, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        #self.dqs4.delete( 0, tkinter.END)
        #self.dqs4.insert( 0, '0.10')
        #self.dqs4.pack(padx=0,pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label( f023, text = " Qmin=0.0 Qmax=4.0")
        #rd.pack(padx=0,pady=0, side = tkinter.LEFT)
        #f023.pack(padx=0,pady=2, fill = tkinter.BOTH)
        self.dqs4 = self.frame.dsp_Si311
        self.dqs4.setMaximum(9.99)
        self.dqs4.setMinimum(0.01)
        self.dqs4.setSingleStep(0.01)
        self.dqs4.setValue(0.05)

        #f02.pack(padx=0, pady=2, side = tkinter.LEFT)
        #f0.pack(padx=5, pady=5)

        #f1 = tkinter.LabelFrame(self, text = 'Run No.')
        #rd = tkinter.Label(f1, text = "       ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.run = tkinter.Spinbox( f1, from_=1000, to=50000, increment=1, width=5)
        #self.run.delete( 0, tkinter.END)
        #self.run.insert( 0, '2000')
        #self.run.pack(pady=2,side = tkinter.LEFT)
        self.run = self.frame.run
        self.run.setMaximum(100000)
        self.run.setMinimum(1000)
        self.run.setSingleStep(1)
        self.run.setValue(2000)
#T.Yamada start [20260130]
# Inside App.__init__
        self.cb_UseMon = self.frame.cb_UseMon
        self.mon_run = self.frame.mon_run
        # Disable the spinbox by default
        self.mon_run.setEnabled(False)
        # Connect toggle logic
        self.cb_UseMon.stateChanged.connect(self.toggle_mon_input)
        self.mon_run = self.frame.mon_run
        self.mon_run.setMaximum(100000)
        self.mon_run.setMinimum(1000)
        self.mon_run.setSingleStep(1)
        if FastChopperUse:
            self.mon_run.setValue(24939)#Monitor Stick 225Hz 3cm-3cm Att3
        else:
            self.mon_run.setValue(24487)#Monitor Stick for white. HV 1kV att0 Si111 LambdaCen 6p321 Kicker = 90000, timeout =
        #rd = tkinter.Label( f1, text = "    Temp. Sensor")
        #rd.pack(pady=0, side = tkinter.LEFT)
#.T.Yamada start
        ChList = ("A","B","C","C2","D","D2","D3","D4","D5","C_TC","D_TC","PEACE_U","PEACE_M","PEACE_L","PEACE_UB","PEACE_MB","PEACE_LB")
        #self.Ch = tkinter.StringVar()
        #self.Ch.set(ChList[6])
        #self.ChMenu = tkinter.OptiponMenu( f1, self.Ch, *ChList)
        #self.ChMenu.pack(pady=2, side = tkinter.LEFT)
        self.Ch = self.frame.Ch
#T.Yamada End
        #button2 = tkinter.Button( f1, text = "Read Temp.", command = self.readtemp)
        #button2.pack( side = tkinter.LEFT)
        self.frame.bt_ReadTemp.clicked.connect(self.readtemp)
        #f1.pack(padx=5, pady=5, fill = tkinter.BOTH)
        #f2 = tkinter.LabelFrame(self, text = 'Parameter')
        #f20 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        #rd = tkinter.Label( f20, text="λcenter ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.lam = tkinter.Spinbox( f20, from_=0, to=99.999, increment=0.001, width=6, format='%6.3f')
        #self.lam.delete( 0, tkinter.END)
        self.lam = self.frame.lam
        self.lam.setMaximum(99.999)
        self.lam.setMinimum(0.000)
        self.lam.setDecimals(3)
        self.lam.setSingleStep(0.001)
# 
        if ( FastChopperUse == True):
            #self.lam.insert( 0, ' 6.321')
            self.lam.setValue(6.321)
        else:
            #self.lam.insert( 0, ' 6.321')
            self.lam.setValue(6.321)

        #self.lam.pack(pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label( f20, text="[A]/ dλ ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.dl = tkinter.Spinbox( f20, from_=0, to=9.999, increment=0.001, width=5, format='%5.3f')
        #self.dl.delete( 0, tkinter.END)
        #self.dl.insert( 0, '4.15')
        #self.dl.pack(pady=0, side = tkinter.LEFT)
        self.dl = self.frame.dl
        self.dl.setMaximum(9.999)
        self.dl.setMinimum(0.000)
        self.dl.setDecimals(3)
        self.dl.setSingleStep(0.001)
        self.dl.setValue(4.15)

        #rd = tkinter.Label( f20, text="[A]")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #f20.pack(padx=5, pady=2, fill = tkinter.BOTH)

        #f21 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        #rd = tkinter.Label( f21, text="Emin ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.emin = tkinter.Spinbox( f21, from_=-99, to=999, increment=1, width=4, format='%3.0f')
        #self.emin.delete( 0, tkinter.END)
        self.emin = self.frame.emin
        self.emin.setMaximum(100000)
        self.emin.setMinimum(-100000)
        self.emin.setSingleStep(1)
# 
        if ( FastChopperUse == True):
            #self.emin.insert( 0, '-40')
            self.emin.setValue(-40)
        else:
            #self.emin.insert( 0, '-500')
            self.emin.setValue(-500)

        #self.emin.pack(pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label( f21, text="μeV/ Emax ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.emax = tkinter.Spinbox( f21, from_=0, to=999, increment=1, width=4, format='%3.0f')
        #self.emax.delete( 0, tkinter.END)
        self.emax = self.frame.emax
        self.emax.setMaximum(40000)
        self.emax.setMinimum(-40000)
        self.emax.setSingleStep(1)
# 
        if ( FastChopperUse == True):
            #self.emax.insert( 0, '100')
            self.emax.setValue(100)
        else:
            #self.emax.insert( 0, '1500')
            self.emax.setValue(1500)

        #self.emax.pack(pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label( f21, text="μeV/ dE ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.de = tkinter.Spinbox( f21, from_=0.01, to=99, increment=0.01, width=5, format='%5.2f')
        #self.de.delete( 0, tkinter.END)
        self.de = self.frame.de
        self.de.setMaximum(99.0)
        self.de.setMinimum(0.01)
        self.de.setSingleStep(0.01)
# 
        if ( FastChopperUse == True):
            #self.de.insert( 0, ' 0.50')
            self.de.setValue(0.50)
        else:
            #self.de.insert( 0, ' 2')
            self.de.setValue(2.0)

        #self.de.pack(pady=0, side = tkinter.LEFT)
        #rd = tkinter.Label(f21, text="μeV")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #f21.pack(padx=5, pady=2, fill = tkinter.BOTH)
#T. Yamada Add[20150601]
        #self.Ebin_flag = tkinter.BooleanVar()
        self.Ebin_flag = False
        #self.EbinFile_name = "not selected"
        
        #f3=tkinter.LabelFrame(self, text='Detector Efficiency Correction')
        #f21_1 = tkinter.LabelFrame(f2, relief = tkinter.FLAT)
        #cb = tkinter.Checkbutton(f21_1, text='User difined E Bin',variable = self.Ebin_flag).pack(padx=2, pady=2, side = tkinter.LEFT)
        #button5 = tkinter.Button(f21_1, text="Select file", command = self.loadEbinfile).pack(padx=2, pady=2, side = tkinter.LEFT)
        self.frame.cb_UserDefinedEBin.stateChanged.connect(self.changedUserDefinedEBin)
        self.frame.bt_UserDefinedEBin.clicked.connect(self.loadEbinfile)
        ##f21_2 = tkinter.Frame(f2,relief=tkinter.FLAT)
        #self.cb21_3 = tkinter.Label(f21_1, text = self.EbinFile_name)
        #self.cb21_3.pack(padx = 20, pady=2,side = tkinter.LEFT)
        ##f21_2.pack(padx=0, pady=2, fill = tkinter.BOTH)
        #f21_1.pack(padx=0, pady=2, fill = tkinter.BOTH)
        self.EbinFile_name = self.frame.txt_EbinFileName # "txt_EbinFileName" <- self.cb21_3  => self.EbinFile_name
#T. Yamada End [20150601]

        #f22 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        #rd = tkinter.Label( f22, text = "T0Shift ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.T0S = tkinter.Spinbox( f22, from_=0, to=99999, increment=1, width=5, format='%5.0f')
        #self.T0S.delete( 0, tkinter.END)
        self.T0S = self.frame.T0S
        self.T0S.setMaximum(99999)
        self.T0S.setMinimum(0)
        self.T0S.setSingleStep(1)
# 
        if ( FastChopperUse == True):
            #self.T0S.insert( 0, '12325')
            self.T0S.setValue(12325)
        else:
            #self.T0S.insert( 0, '120')
            self.T0S.setValue(120)

        #self.T0S.pack(pady=2, side = tkinter.LEFT)
        #rd = tkinter.Label( f22, text="[μs] ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #f22.pack(padx=5, pady=2, fill = tkinter.BOTH)

        #f23 = tkinter.LabelFrame( f2, relief = tkinter.FLAT)
        #rd = tkinter.Label( f23, text = "StartTime")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.Sta = tkinter.Spinbox( f23, from_=0, to=9999999, increment=1, width=7, format='%7.0f')
        #self.Sta.pack(pady=0, side = tkinter.LEFT)
        self.Sta = self.frame.Sta
        self.Sta.setMaximum(9999999)
        self.Sta.setMinimum(0)
        self.Sta.setSingleStep(1)
        #rd = tkinter.Label( f23, text = "[sec.]/ EndTime ")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #self.End = tkinter.Spinbox( f23,from_=0,to=9999999,increment=1,width=7,format='%7.0f')
        #self.End.delete( 0, tkinter.END)
        #self.End.insert( 0, '9999999')
        #self.End.pack(pady=0, side = tkinter.LEFT)
        self.End = self.frame.End
        self.End.setMaximum(9999999)
        self.End.setMinimum(0)
        self.End.setSingleStep(1)
        self.End.setValue(1000000)
        #rd = tkinter.Label( f23, text = "[sec.]")
        #rd.pack(pady=0, side = tkinter.LEFT)
        #f23.pack(padx=5,pady=2, fill = tkinter.BOTH)
        #f2.pack(padx=5, pady=5, fill = tkinter.BOTH)

#T. Yamada Add[20140524]
        #self.DetEffCr_flag = tkinter.BooleanVar()
        #self.DetEffFile_name = "not selected"
        self.DetEffCr_flag = False
        self.DetEffFile_name = self.frame.txt_VanadiumCorr
        
        #f3=tkinter.LabelFrame(self, text='Detector Efficiency Correction')
        #f31 = tkinter.LabelFrame(f3, relief = tkinter.FLAT)
        #cb = tkinter.Checkbutton(f31, text='Vanadium Correction',variable = self.DetEffCr_flag).pack(padx=2, pady=2, side = tkinter.LEFT)
        #button4 = tkinter.Button(f31, text="Select file", command = self.loadfile).pack(padx=2, pady=2, side = tkinter.LEFT)
        self.frame.cb_VanadiumCorr.stateChanged.connect(self.changedVanadiumCorr)
        self.frame.bt_VanadiumCorr.clicked.connect(self.loadfile)
        #f31.pack(padx=0, pady=2, fill = tkinter.BOTH)
        ##f32 = tkinter.Frame(f3,relief=tkinter.FLAT)
        #self.cb31 = tkinter.Label(f31, text = self.DetEffFile_name)
        #self.cb31.pack(padx = 20, pady=2,side = tkinter.LEFT)
        ##f32.pack(padx=0, pady=2, side = tkinter.LEFT)
        #f3.pack(padx=5, pady=5, fill = tkinter.BOTH)
#T. Yamada End [20140524]
#T. Yamada Add [20141110]
        ##self.DoMask_flag = tkinter.BooleanVar()
        #self.MaskFile_name = "maskTY.txt"
        self.MaskFile_name = self.frame.txt_MaskFile
        
        #f4=tkinter.LabelFrame(self, text='Select mask file')
        #f41 = tkinter.LabelFrame(f4, relief = tkinter.FLAT)
        ##cb = tkinter.Checkbutton(f41, text='Vanadium Correction',variable = self.DetEffCr_flag).pack(padx=2, pady=2, side = tkinter.LEFT)
        #button4 = tkinter.Button(f41, text="Select file", command = self.loadMaskfile).pack(padx=2, pady=2, side = tkinter.LEFT)
        self.frame.bt_MaskFile.clicked.connect(self.loadMaskfile)
        #f41.pack(padx=0, pady=2, fill = tkinter.BOTH)
        #f42 = tkinter.Frame(f4,relief=tkinter.FLAT)
        #self.cb41 = tkinter.Label(f41, text = self.MaskFile_name)
        #self.cb41.pack(padx = 20, pady=2,side = tkinter.LEFT)
        #f41.pack(padx=0, pady=2, fill = tkinter.BOTH)
        ##f42.pack(padx=0, pady=2, side = tkinter.LEFT)
        #f4.pack(padx=5, pady=5, fill = tkinter.BOTH)
#T. Yamada End [20141110]

        #button1 = tkinter.Button(self, text = "Launch Data Reduction", command = self.launch)
        #button3 = tkinter.Button(self, text = "Quit", command = self.master.destroy)
        #button1.pack(side = tkinter.LEFT)
        #button3.pack(side = tkinter.RIGHT)
        #self.pack()
        self.frame.bt_LaunchDataRed.clicked.connect(self.launch)
        self.frame.bt_Quit.clicked.connect(self.close)
# self.step == 10: launch stand-by
        self.step = 11
        self.show()

    def toggle_mon_input(self, state):
        """Enable or disable the Monitor Run input based on checkbox state."""
        if self.cb_UseMon.isChecked():
            self.mon_run.setEnabled(True)
        else:
            self.mon_run.setEnabled(False)

    def changedUserDefinedEBin(self, evt=None):
        if self.frame.cb_UserDefinedEBin.checkState() == QtCore.Qt.Checked:
            self.Ebin_flag = True
        else:
            self.Ebin_flag = False
    def changedVanadiumCorr(self, evt=None):
        if self.frame.cb_VanadiumCorr.checkState() == QtCore.Qt.Checked:
            self.DetEffCr_flag = True
        else:
            self.DetEffCr_flag = False
    def loadfile(self):
        def _getcorrefftype(fname):
            f = open(fname,"r")
            line = f.readline()
            f.close()
            ret = 0
            if line.find(",") != -1:
                ret = len(line.split(",")) #ret = 120
            else:
                ret = len(line.split(" ")) #ret = 121
            
            if ret != 120 and ret !=121:
                print ("File is incorrect. Check the file {0}, {1}".format(fname, ret))

            return ret
        
        self.path_name = "/home/dnaadmin/ana/DNA"
        tmp_path = MU.UtsusemiEnvGetUserDir()
        if tmp_path != "":
            self.path_name = os.path.join(tmp_path, "ana", "DNA")
        #self.DetEffFile_name = tkFileDialog.askopenfilename(initialdir = self.path_name)
        tmp_filepath, filt = QtWidgets.QFileDialog.getOpenFileName(self, "Open File ...", self.path_name)
        if tmp_filepath == "":
            return
        else:
            self.DetEffFile_name.setText(tmp_filepath)
        #self.cb31.configure(text=self.DetEffFile_name)
        #self.DetEffFileType = _getcorrefftype(self.DetEffFile_name)
        self.DetEffFileType = _getcorrefftype(tmp_filepath)
        
        #print (self.DetEffFile_name, self.DetEffCr_flag.get(),self.DetEffFileType)
        print (tmp_filepath, self.DetEffCr_flag, self.DetEffFileType)
    

    def loadMaskfile(self):
        self.path_name = "/home/dnaadmin/ana/DNA/Mask"
        tmp_path = MU.UtsusemiEnvGetUserDir()
        if tmp_path != "":
            self.path_name = os.path.join(tmp_path, "ana", "DNA", "Mask")
        #self.MaskFile_name = tkFileDialog.askopenfilename(initialdir = self.path_name,filetypes=[('mask file','.txt')])
        tmp_filepath, filt = QtWidgets.QFileDialog.getOpenFileName(self, "Open Mask File ...", self.path_name, filter='Mask file (*.txt *.xml)')
        if tmp_filepath == "":
            return
        else:
            tmp_base_name = os.path.basename(tmp_filepath)
            self.MaskFile_name.setText(tmp_base_name)
        #self.MaskFile_name = os.path.basename(self.MaskFile_name)
        #self.cb41.configure(text=self.MaskFile_name)
        print (self.MaskFile_name.text())
       
    def loadEbinfile(self):
        self.path_name = "/home/dnaadmin/"
        tmp_path = MU.UtsusemiEnvGetUserDir()
        if tmp_path != "":
            self.path_name = tmp_path
        #self.EbinFile_name = tkFileDialog.askopenfilename(initialdir = self.path_name)
        tmp_filepath, filt = QtWidgets.QFileDialog.getOpenFileName(self, "Open Ebin File ...", self.path_name)
        #self.cb21_3.configure(text=self.EbinFile_name)
        if tmp_filepath == "":
            return
        else:
            self.EbinFile_name.setText(tmp_filepath)


    def sel_si(self):
        if self.step == 11:
            #self.si = self.v.get()
            self.si = self.rb_group.checkedId() - 1
            if self.si != 2:
                #self.MaskFile_name = "maskTY.txt"
                #self.cb41.configure(text=self.MaskFile_name)
                self.MaskFile_name.setText("maskTY.txt")
            elif self.si == 2:
                #self.MaskFile_name = "mask311.txt"
                #self.cb41.configure(text=self.MaskFile_name)
                self.MaskFile_name.setText("mask311.txt")

    def readtemp(self):
        if self.step == 11:
            #self.runNo = int(self.run.get())
            self.runNo = int(self.run.value())
            self.mon_runNo = int(self.mon_run.value())
            #TCh = self.Ch.get()
            TCh = self.Ch.currentText()
            print ("RunNo:{0:d}, TempCh: {1}".format(self.runNo,TCh))
            #get Run No.
            print ("SQWGUI--> Reading Temperature start")
            EC = Cmm.CheckSampleTempByRunNumber( self.runNo,TCh)
            print ("SQWGUI--> Reading Temperature end")
            MP.MPlot(EC)
            #self.errorstr.set("Temperature run# %d Ch %s" % (self.runNo, TCh))
            self.errorstr.setText("Temperature run# %d Ch %s" % (self.runNo, TCh))

    def launch(self):
        print ("koko283", self.step)
        if self.step == 11:
            self.mydatetime = datetime.datetime.now()
            datetimestr = self.mydatetime.strftime('%Y/%m/%d %H:%M:%S')
            datestr = self.mydatetime.strftime('Utsusemi%Y%m%d.txt')
            #Get Run No.
            #self.runNo = int(self.run.get())
            self.runNo = int(self.run.value())

            #Get Q parameter
            self.Qmin = 0.0
            if self.si == 0:#Si(111)
                #self.DQ = float(self.dqs1.get())
                self.DQ = float(self.dqs1.value())
                self.Qmax = 2.0
            elif self.si == 1:#Si(333)
                #self.DQ = float(self.dqs3.get())
                self.DQ = float(self.dqs3.value())
                self.Qmax = 6.0
            elif self.si == 2:
                #self.DQ = float(self.dqs4.get())
                self.DQ = float(self.dqs4.value())
                self.Qmax = 4.0
            
            QParam = "%.1f,%.3f,%.3f" %(self.Qmin,self.Qmax,self.DQ)	
            print ("->>QParam: {0}".format(QParam))

            #Get Lambda Parameter
            # LambdaCenter = float(self.lam.get())
            # dLambda = float(self.dl.get())
            #self.LambdaParam = "%.4f,%.4f" %(float(self.lam.get()),float(self.dl.get()))
            self.LambdaParam = "%.4f,%.4f" %(float(self.lam.value()),float(self.dl.value()))
            print ("->>LambdaParam: {0}".format(self.LambdaParam))

            #Get HWParam
            # Emin = float(self.emin.get())/1000.0
            # Emax = float(self.emax.get())/1000.0
            # dE = float(self.de.get())/1000.0
            #self.HWParam = "%.5f,%.5f,%.5f" %((float(self.de.get())/1000.0),(float(self.emin.get())/1000.0),(float(self.emax.get())/1000.0))
            self.HWParam = "%.5f,%.5f,%.5f" %((float(self.de.value())/1000.0),(float(self.emin.value())/1000.0),(float(self.emax.value())/1000.0))
            print ("->>HWParam: {0}".format(self.HWParam))

            #Get T0Shift
            #self.T0Shift = float(self.T0S.get())
            self.T0Shift = float(self.T0S.value())
            print ("->>T0Shift: {0}".format( self.T0Shift))

            #Get Time Parameter
            # startTime = int(self.Sta.get())
            # endTime = int(self.End.get())
            #self.TimeParam = "%d,%d" %(int(self.Sta.get()), int(self.End.get()))
            self.TimeParam = "%d,%d" %(int(self.Sta.value()), int(self.End.value()))
            print ("->>TimeParam: {0}".format(self.TimeParam))

            #Write log file
            self.fp = open( "%s" % (datestr), "a")
            self.fp.write( "%s run:%d Si" % (datetimestr, self.runNo))
            if self.si == 0:
                #self.fp.write( "(111):de:%s min:%3.1f max:%3.1f" % (self.dqs1.get(),0,2.0))
                self.fp.write( "(111):de:%s min:%3.1f max:%3.1f" % (self.dqs1.value(),0,2.0))
            elif self.si == 1:
                #self.fp.write( "(333):de:%s min:%3.1f max:%3.1f" % (self.dqs3.get(),0,6.0))
                self.fp.write( "(333):de:%s min:%3.1f max:%3.1f" % (self.dqs3.value(),0,6.0))
            elif self.si == 2:
                #self.fp.write( "(311):de:%s min:%3.1f max:%3.1f" % (self.dqs4.get(),0,6.0))
                self.fp.write( "(311):de:%s min:%3.1f max:%3.1f" % (self.dqs4.value(),0,6.0))
            #self.fp.write(" lambda:%s dlambda:%s" % (self.lam.get(), self.dl.get()))
            self.fp.write(" lambda:%6.3f dlambda:%5.3f" % (self.lam.value(), self.dl.value()))
            #self.fp.write(" emin:%s emax:%s de:%s" % (self.emin.get(), self.emax.get(),self.de.get()))
            self.fp.write(" emin:%3.0f emax:%3.0f de:%5.2f" % (self.emin.value(), self.emax.value(),self.de.value()))
            #self.fp.write(" T0shift:%s" % (self.T0S.get()))
            self.fp.write(" T0shift:%5.0f" % (self.T0S.value()))
            #self.fp.write(" Start:%s End:%s" % (self.Sta.get(), self.End.get()))
            self.fp.write(" Start:%7.0f End:%7.0f" % (self.Sta.value(), self.End.value()))
            #self.fp.write(" Maskfile:%s" % (self.MaskFile_name))
            self.fp.write(" Maskfile:%s" % (self.MaskFile_name.text()))
            #self.fp.write(" EfficiencyFile:%s %s\n" % ( self.DetEffFile_name, self.DetEffCr_flag.get()))
            if self.DetEffCr_flag:
                self.fp.write(" EfficiencyFile:%s True\n" % ( self.DetEffFile_name.text()))
            else:
                self.fp.write(" EfficiencyFile:%s False\n" % ( self.DetEffFile_name.text()))
            self.fp.close()

            self.step = 0
            #self.errorstr.set("Check Data  (0/11)")
            self.errorstr.setText("Check Data  (0/11)")
            #self.after( 100, self.countup)
            self.timer = QtCore.QTimer(self)
            self.timer.timeout.connect(self.countup)
            self.timer.start(100)

    def countup(self):
        print ("TY1", self.step)
        if self.step == 0:
            copyresult, messg = CopyEventData(int(self.runNo))
            #self.errorstr.set(messg)
            self.errorstr.setText(messg)
            if copyresult == 1:
                self.step = 10
                print ("TY2", copyresult, messg, self.step)
            else:
                #self.errorstr.set("GetMonHistogram.  (1/11)")
                print ("TY3", copyresult, messg, self.step)
                self.errorstr.setText("GetMonHistogram.  (1/11)")
                print ("TY3_1",self.runNo, self.LambdaParam, self.T0Shift, 0.0,  self.TimeParam, FastChopperUse)  
                if self.cb_UseMon.isChecked():#20260130 Yamada add
                    print("Using Monitor Stick")
                    self.mon_runNo = int(self.mon_run.value())
                    self.EC = Cmm.GetHistogramMonAtSmp(self.mon_runNo, True, self.LambdaParam, self.T0Shift, 0.0, False, "0/1000000", FastChopperUse,False)  
#                    self.EC = Cmm.GetHistogramMon(self.mon_runNo, True, self.LambdaParam, self.T0Shift, 0.0, False, self.TimeParam, FastChopperUse)  
                else:
                    self.EC = Cmm.GetHistogramMon(self.runNo, True, self.LambdaParam, self.T0Shift, 0.0, False, self.TimeParam, FastChopperUse)  
                    Cmm.MutiplyConstant(self.EC, 1e-09)
                Cmm.OutputECText( self.EC, "TY")
                #self.errorstr.set("MultiplyConstant. (2/11)")
                self.errorstr.setText("MultiplyConstant. (2/11)")
                self.step = 1
        elif self.step == 1:
            if self.si == 0:
                #self.errorstr.set("GetHistogramHW.   (3/11)")
                self.errorstr.setText("GetHistogramHW.   (3/11)")
            elif self.si == 1:
                #self.errorstr.set("GetHistogramHW333.(3/11)")
                self.errorstr.setText("GetHistogramHW333.(3/11)")
            elif self.si == 2:
                #self.errorstr.set("GetHistogramHW311.(3/11)")
                self.errorstr.setText("GetHistogramHW311.(3/11)")
            self.step = 2
        
        elif self.step == 2:
            #if self.Ebin_flag.get():
            if self.Ebin_flag:
                self.HWParam = MDCR._readBinInfo(self.EbinFile_name.text(),"\n",",")
                #print "koko", self.HWParam
                #print "kokoko"
                #return -1

            if self.si == 0:
                self.DAT = Cmm.GetHistogramHW( self.runNo, self.HWParam, self.LambdaParam, self.T0Shift, False, self.TimeParam, FastChopperUse)
            elif self.si == 1:
                self.DAT = Cmm.GetHistogramHW333( self.runNo, self.HWParam, self.LambdaParam, self.T0Shift, False, self.TimeParam, FastChopperUse)
            elif self.si == 2:
                self.DAT = Cmm.GetHistogramHW311( self.runNo, self.HWParam, self.LambdaParam, self.T0Shift, False, self.TimeParam, FastChopperUse)
            #self.errorstr.set("DoMask.           (4/11)")
            self.errorstr.setText("DoMask.           (4/11)")
            if self.cb_UseMon.isChecked(): #20260130 Yamada Add
                factorBC = 1e8
                Cmm.NormByBeamCurrent(self.DAT,factorBC,0.0,True)
            self.step = 3
        elif self.step == 3:
            """
            [20141110 Yamada Modified]
            if self.si != 2:
                Cmm.DoMask(self.DAT, "maskTY.txt")
            else:
                Cmm.DoMask(self.DAT, "mask311.txt")
            """
            Cmm.DoMask(self.DAT, self.MaskFile_name.text())
            #self.errorstr.set("ILambdaCorrDNA.   (5/11)")
            self.errorstr.setText("ILambdaCorrDNA.   (5/11)")
            self.step = 4
        elif self.step == 4:
            if self.cb_UseMon.isChecked(): #20260130 Yamada Add
                if FastChopperUse:
                    AttUse = True
                    monDatFile = "monitor_data_stick_highreso.dat"
                else:
                    AttUse = False
                    monDatFile = "monitor_data_stick_highflux.dat"
                self.ECM = Cmm.ILambdaCorrUseMonAtSmpDNA(dat=self.DAT,ec=self.EC,monDatFile=monDatFile,isHighReso=AttUse,attenuateVal=0.0) #isHighReso=True means using Att, False means not using Att. If other attenuator is used for monitor stick, attenuateVal should be changed. The default (0.0) is calculated by Dr. Matsuura.
                print("Using Monitor Stick")
            else:
                self.ECM = Cmm.ILambdaCorrDNA( self.DAT, self.EC, True)
            #self.errorstr.set("SolidAngleCorrDNA.(6/11)")
            self.errorstr.setText("SolidAngleCorrDNA.(6/11)")
            self.step = 5
        elif self.step == 5:
            #if self.DetEffCr_flag.get() and self.DetEffFileType == 120:
            if self.DetEffCr_flag and self.DetEffFileType == 120:
                self.ECM2 = Cmm.SolidAngleCorrDNA( self.ECM, True, False, False, "sample", DetEffDataPath=self.DetEffFile_name.text() )
                self.ECM2 = EneOrig.DoEfficiencyCorrection(dat=self.ECM2,filename=self.DetEffFile_name.text(),mode=0)
            else:
                self.ECM2 = Cmm.SolidAngleCorrDNA( self.ECM, True, False, self.DetEffCr_flag, "sample", DetEffDataPath=self.DetEffFile_name.text() )
            #self.errorstr.set("MultiplyConstant. (7/11)")
            self.errorstr.setText("MultiplyConstant. (7/11)")
            self.step = 6
        elif self.step == 6:
            if self.cb_UseMon.isChecked(): #20260130 Yamada Add
                if FastChopperUse:
                    Cmm.MutiplyConstant(self.ECM2, 1e1)
                else:
                    Cmm.MutiplyConstant(self.ECM2, 1e2)
            else:
                Cmm.MutiplyConstant( self.ECM2, 1e-06)
            #self.errorstr.set("CreateQEMap.      (8/11)")
            self.errorstr.setText("CreateQEMap.      (8/11)")
            self.step = 7
        elif self.step == 7:
            Cmm.KiKfCorrection(self.ECM2)
            #self.errorstr.set("KiKfCorrection.      (9/11)")
            self.errorstr.setText("KiKfCorrection.      (9/11)")
            self.step = 8
        elif self.step == 8:
            self.ECA = Cmm.CreateQEMap( self.ECM2, self.Qmin, self.Qmax, self.DQ)
            #self.errorstr.set("OutoutECA.        (10/11)")
            self.errorstr.setText("OutoutECA.        (10/11)")
            self.step = 9
        elif self.step == 9:
            footer = "_GUIQE"
            if self.DetEffCr_flag:
                footer = "_VGUIQE"
            Cmm.OutputECA( self.ECA, footer)
            Cmm.OutputECAText( self.ECA, footer)
            Cmm.OutputECADAVEText( self.ECA, footer)
            #T.Yamada Add[20150601]
            #if self.Ebin_flag.get():
            if self.Ebin_flag:
                Cmm.OutputNeiECAText(self.ECA,"_neimMDRQE")
            #T.Yamada End[20150601]
            
            #self.errorstr.set("M2PLOT(ECA)      (11/11)")
            self.errorstr.setText("M2PLOT(ECA)      (11/11)")
            self.step = 10
        elif self.step == 10:
            MP2.M2PlotPlusCui(self.ECA).ShowData()
            #self.errorstr.set("M2PLOT end. run# %d" % (self.runNo))
            self.errorstr.setText("M2PLOT end. run# %d" % (self.runNo))
            self.step = 11
        #if self.step < 11:
        #    self.after(200, self.countup)
        if self.step == 11:
            self.timer.stop()

def CopyEventData(RunNo):
    #if RunNo < 14082:
    if RunNo < 0:
        command = "ssh dnaadmin@192.168.1.10 /home/dnaadmin/Yamada/01_CopyEventData/CopyEventEdata.sh %d" %(RunNo)
        args = command.split(" ")
        proc = subprocess.Popen(args, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        stdout_data, stderr_data = proc.communicate()
        print ("stdout_data >> {0}".format(stdout_data.decode('utf-8')))
        print ("stderr_data >> {0}".format(stderr_data.decode('utf-8').replace("\n","")))
        result = proc.returncode
        #proc.close()
        if result == 1:
            messg =  "DNA%06d_* is not found on /data and /ifs_data/DNA/DNA" %RunNo
        else:
            messg =  "DNA%06d_* is ready for the reduction." %RunNo
    else:
        result = 0
        messg =  "DNA%06d_* is ready for the reduction." %RunNo

    return result, messg

if __name__ == "__main__":

    if ( os.environ.get("FastChopperUse")=="True"):
        FastChopperUse = True
    else:
        FastChopperUse = False
    if PYSIDEVER == 2:
        if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
            QtWidgets.QApplication.setAttribute(
                QtCore.Qt.AA_EnableHighDpiScaling, True)

    #app = App()
    #app.mainloop()
    app = QApplication(sys.argv)
    win = App(None)
    if PYSIDEVER == 6:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding: utf-8 -*-

"""
ElementContainerArrayの中のContainerごとの総和（積分ではない）
ElementContainerMatrix中のArrayを指定して取り出し、
そのArray中の各ContainerのKyeY成分の和を計算し
そのContainerのindexをKeyX、総和をKeyYとする
新しいElementContainerを作り出す。
"""
from __future__ import print_function
import Manyo as mm
import math

##############################


class PutSum(object):
    ##############################
    def __init__(self, dat=None):
        """
        コンストラクタ

        @param  dat(ElementContainerMatrix)
        """
        if dat is not None:
            # if type(dat)!=type(mm.ElementContainerMatrix()):
            if not isinstance(dat, mm.ElementContainerMatrix):
                return
        self.KeyY = "Intensity"
        self.dat = dat

    ##############################
    def PutSumEC(self, psd, tmin=0.0, tmax=39500.0):
        """
        KeyYの和を算出
        与えられたElementContainerMatrixから指定されたindexのArrayを取り出し、
        指定したTOF領域で和を行った結果をElementContainerへ出力する。
        @param  psd(UInt4/ElementContainerArray)  index of Array
        @param  tmin(Double) minimum TOF of range for summation.
        @param  tmax(Double) maximum TOF of range for summation.
        @retval EC(ElementContainer)
        """
        if isinstance(psd, int):
            if self.dat is None:
                print("You should input ElementContainer Array!!")
                return
            if psd >= self.dat.PutTableSize():
                print("PSD number is to high!")
                return

            a_psd = self.dat.Put(psd)
        # elif type(psd)==type(mm.ElementContainerArray()):
        elif isinstance(psd, mm.ElementContainerArray):
            a_psd = psd
        else:
            print("You should input ElemnetContainer Array!!")
            return

        num_pixel = a_psd.PutTableSize()

        pixel_list = []
        pixel_sums = []
        pixel_errs = []

        tof_vec = a_psd.PutPointer(0).PutX()
        tof = []
        for t in range(tof_vec.size() - 1):
            tof.append((tof_vec[t] + tof_vec[t + 1]) / 2.0)

        index_min = -1
        index_max = -1
        for t in range(len(tof)):

            if (index_min == -1) and (tmin < tof[t]):
                index_min = t

            if (index_max == -1) and (tmax < tof[t]):
                index_max = t

        if index_min == -1:
            index_min = 0
        if index_max == -1:
            index_max = len(tof) - 1

        print("index min, max = {}".format(index_min, index_max))

        for i in range(num_pixel):
            pixel_list.append(i - 0.5)
            a_sum, a_err = self._SumEC(a_psd.Put(i), index_min, index_max)
            pixel_sums.append(a_sum)
            pixel_errs.append(a_err)

        pixel_list.append(num_pixel + 0.5)

        head_psd = a_psd.PutHeader()
        head_EC = mm.HeaderBase()
        if head_psd.CheckKey("PSDID") != 0:
            head_EC.Add("PSDID", head_psd.PutInt4("PSDID"))
        else:
            if isinstance(psd, int):
                head_EC.Add("PSDID", psd)
            else:
                head_EC.Add("PSDID", 0)
        EC = mm.ElementContainer(head_EC)
        EC.Add("PSD", pixel_list)
        EC.Add("Intensity", pixel_sums)
        EC.Add("Error", pixel_errs)
        EC.SetKeys("PSD", "Intensity", "Error")

        return EC

    ##############################
    def _SumEC(self, a_pixel, index_min, index_max):
        """
        ElementContainerのKeyYの和
        index_minからindex_maxまでのKeyYの総和を返す
        @param a_pixel(ElementContainer) 対象となるElementContainer
        @param index_min(UInt4) 和をとる領域の最小index
        @param index_max(UInt4) 和をとる領域の最大index
        @retval tupple of sum and err (Double) 総和 and error
        """
        intensity = a_pixel.PutY()
        error = a_pixel.PutE()
        a_sum = 0.0
        a_err = 0.0
        for t in range(index_min, index_max):
            a_sum = a_sum + intensity[t]
            a_err = a_err + error[t] * error[t]

        return (a_sum, math.sqrt(a_err))

    ##############################
    def PutSumText(self, psd, tmin=0.0, tmax=39500.0, filename="PutSum.txt"):
        """
        KeyYの和をTEXTファイルに出力
        与えられたElementContainerMatrixから指定されたindexのArrayを取り出し、
        指定したTOF領域で和を行った結果をTEXTファイルに出力。戻り値にはそのElementCotainer。
        @param   psd(UInt4)  index of Array
        @param  tmin(Double) minimum TOF of range for summation.
        @param  tmax(Double) maximum TOF of range for summation.
        @param  filename(String) File name of output
        @retval EC(ElementContainer)
        """
        EC2txt = self.PutSumEC(psd, tmin, tmax)

        wf = open(filename, "w")

        pixel_list = EC2txt.PutX()
        pixel_sums = EC2txt.PutY()
        xx = []
        for i in range(len(pixel_list) - 1):
            xx.append((pixel_list[i] + pixel_list[i + 1]) / 2)

        for i in range(len(pixel_sums)):
            st = "%d,%f\n" % (int(xx[i]), pixel_sums[i])
            wf.write(st)

        wf.close()

        return EC2txt



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

from xml.dom import minidom, Node
import utsusemi.SAS.ana.Reduction.AtomDBTaikan as ADB
import Manyo.Utsusemi as mu
import os

###############################################################################
# 定数値定義
###############################################################################
# 検出器効率計算用パラメータ
ABS_WAVE_LENGTH = 1.8		# 1.8[A](25.25meV) absorption cross section for 2200 m/s neutrons

N_A = 6.02214129e23			# [mol^-1] Avogadro constant
R_GAS_CONST = 8.20574e-2	# [(L*atm)/(K*mol)]molar gas constant

###############################################################################
# 検出器構成情報基底クラス
###############################################################################
class EffPrmInfoCommon:
	"""
		検出器の構成情報を保持するクラス
		属性
		name				: (str)name
		size				: (float)PSDの半径	[cm]
		outerThick			: (float)[]外殻肉厚リスト	[cm]
	"""
	##############################
	def __init__(self , name="", size=0.0, outerThick=[]):
		"""
		初期化処理(コンストラクタ)
		属性の初期化を行う。
		@param	name				(str)name
		@param	size				(float)PSDの半径	[cm]
		@param	outerThick			(float)[]外殻肉厚リスト	[cm]
		"""
		self.name 					= name
		self.size					= size					# PSDの半径		[cm]
		self.outerThick				= outerThick			# 外殻肉厚リスト		[cm]

	##############################
	def Show(self):
		print("name:%s" % (self.name))
		print("size:%g" % (self.size))
		#print "outerThick:%g" % (self.outerThick)
		print("outerThick:", end=' ')
		print(self.outerThick)


###############################################################################
# 検出器構成情報クラス
###############################################################################
class EffPrmInfoRead(EffPrmInfoCommon):
	"""
		検出器の構成情報を保持するクラス
		属性
		EffPrmInfoCommon	: (EffPrmInfoCommon)検出器構成情報基底クラス
		outerDensity		: (float)外殻密度	[g/cm^3]
		gasPressure			: (float)ガス圧	[atm]
		gasTemperture		: (float)ガス温度	[K]
		outerElement		: (dictionary)外殻構成	key = element, value = ratio
		gasElement			: (dictionary)GAS構成	key = element, value = ratio

	"""
	##############################
	def __init__(self , name="", size=0.0, outerThick=[], outerDensity=0.0, gasPressure=0.0, gasTemperture=0.0, outerElement={}, gasElement={}):
		"""
		初期化処理(コンストラクタ)
		属性の初期化を行う。
		@param	name				(str)name
		@param	size				(float)PSDの半径	[cm]
		@param	outerThick			(float)[]外殻肉厚リスト	[cm]
		@param	outerDensity		(float)外殻密度	[g/cm^3]
		@param	gasPressure			(float)ガス圧	[atm]
		@param	gasTemperture		(float)ガス温度	[K]
		@param	outerElement		(dictionary)外殻構成	key = element, value = wt-ratio
		@param	gasElement			(dictionary)GAS構成	key = element, value = ratio
		"""
		EffPrmInfoCommon.__init__(self, name, size, outerThick)

		self.outerDensity		= outerDensity		# 外殻密度 [g/cm^3]
		self.gasPressure		= gasPressure		# ガス圧 [atm]
		self.gasTemperture		= gasTemperture		# ガス温度 [K]

		self.outerElement		= outerElement		# 外殻構成
		self.gasElement			= gasElement		# GAS構成

	##############################
	def Show(self):
		"""
		クラス情報の表示
		"""
		EffPrmInfoCommon.Show(self)
		print("outerDensity:%g" % (self.outerDensity))
		print("gasPressure:%g" % (self.gasPressure))
		print("gasTemperture:%g" % (self.gasTemperture))

		print("outerElement")
		#print self.outerElement
		for k,v in list(self.outerElement.items()):
			print(k,v)
		print("gasElement")
		#print self.gasElement
		for k,v in list(self.gasElement.items()):
			print(k,v)


###############################################################################
# 計算済み検出器構成情報クラス
###############################################################################
class EffPrmInfoCalc(EffPrmInfoCommon):
	"""
		計算済みの検出器の構成情報を保持するクラス
		属性：
		EffPrmInfoCommon		: (EffPrmInfoCommon)検出器構成情報基底クラス
		outerAbs				: (float)外殻全吸収断面積	[cm^2]
		outerScatt				: (float)外殻全散乱断面積	[cm^2]
		outerNumberDensity		: (float)外殻数密度			[個/cm^3]
		gasAbs					: (float)ガス全吸収断面積	[cm^2]
		gasNumberDensity		: (float)ガス数密度		[個/cm^3]
	"""
	##############################
	def __init__(self , name="", size=0.0, outerThick=[], outerAbs=0.0, outerScatt=0.0, outerNumberDensity=0.0, gasAbs=0.0, gasNumberDensity=0.0):
		"""
		初期化処理(コンストラクタ)
		属性の初期化を行う。
		@param	name				(str)name
		@param	size				(float)PSDの半径	[cm]
		@param	outerAbs			(float)外殻全吸収断面積	[cm^2]
		@param	outerScatt			(float)外殻全散乱断面積	[cm^2]
		@param	outerNumberDensity	(float)外殻数密度			[個/cm^3]
		@param	gasAbs				(float)ガス全吸収断面積	[cm^2]
		@param	gasNumberDensity	(float)ガス数密度		[個/cm^3]
		"""
		# 取得値
		EffPrmInfoCommon.__init__(self, name, size, outerThick)

		self.outerAbs			= outerAbs				# 外殻全吸収断面積	[cm^2]
		self.outerScatt			= outerScatt			# 外殻全散乱断面積	[cm^2]
		self.outerNumberDensity	= outerNumberDensity	# 外殻数密度			[個/cm^3]

		self.gasAbs					= gasAbs				# ガス全吸収断面積	[cm^2]
		self.gasNumberDensity		= gasNumberDensity		# ガス数密度[個/cm^3]

	##############################
	def Show(self):
		"""
		クラス情報の表示
		"""
		EffPrmInfoCommon.Show(self)
		print("outerAbs:%g" % (self.outerAbs))
		print("outerScatt:%g" % (self.outerScatt))
		print("outerNumberDensity:%g" % (self.outerNumberDensity))
		print("gasAbs:%g" % (self.gasAbs))
		print("gasNumberDensity:%g" % (self.gasNumberDensity))


###############################################################################
# 検出器構成情報ファイル読み込み
###############################################################################
class ReadEfficiencyParam:
	"""
	検出器構成情報ファイル読み込み
		属性：
		detecList :(EffPrmInfoRead)[]検出器の構成情報リスト

	***TDB***
		EffPrmInfoReadの数分読み込んでリストを作成するけど、利用するのはid=1
		(idと検出器構成をきちんと紐付けられるようになってないので)
		idは昇順じゃなくても可
		ratioが合計1になってるかどうかはチェックしない
	"""
	##############################
	def __init__(self, strEffPrmInfo = 'psdinfo', strSize = 'radius', strOuter = 'cylinder', numThick = 1):
		"""
		初期化処理(コンストラクタ)
		属性の初期化を行う。
		@param	strEffPrmInfo	(str)検出器効率パラメータ情報のタグ名
										(psd:psdinfo	monitor:monitorinfo)
		@param	strSize			(str)検出器サイズのタグ名
										(psd:radius		monitor:width)
		@param	strOuter		(str)外殻のタグ名
										(psd:cylinder	monitor:shell)
		@param	numThick		(integer)thickの数
										(psd:1			monitor:2)
		"""
		self.detecList = []

		#読み込み
		self.__ReadFile(strEffPrmInfo, strSize, strOuter, numThick)

	##############################
	def __ReadFile(self, strEffPrmInfo, strSize, strOuter, numThick):
		"""
		パラメータファイルの読み込み
		@param	strEffPrmInfo	(str)検出器効率パラメータ情報のタグ名
										(psd:psdinfo	monitor:monitorinfo)
		@param	strSize			(str)検出器サイズのタグ名
										(psd:radius		monitor:width)
		@param	strOuter		(str)外殻のタグ名
										(psd:cylinder	monitor:shell)
		@param	numThick		(integer)thickの数
										(psd:1			monitor:2)
		"""

		#xmlpath = os.path.join(os.environ["MLF_USR_DIR"],"ana/xml","DetectorInfo.xml")
		#xmlpath = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml","DetectorInfo.xml")
                xmlpath = os.path.join(mu.UtsusemiEnvGetInstDir(),"ana","xml","DetectorInfo.xml")
		doc = minidom.parse(xmlpath)

		self.detecList = []
		# タグがstrEffPrmInfoの要素を取得
		for node in doc.getElementsByTagName(strEffPrmInfo):
			# 属性：name, size取得
			name = getAttributeCheckStr(node.getAttribute('name'))
			size = getAttributeCheckFloat(node.getAttribute(strSize))

			#  子要素の初期化
			outerThick = []
			outerDensity = 0.0
			outerElement ={}
			gasPressure = 0.0
			gasTemperture= 0.0
			gasElement ={}

			# 子要素のouterとgasを取得
			for child in node.childNodes:
				if child.nodeType == Node.ELEMENT_NODE:
					if child.tagName == strOuter:
						# 属性：thick, density取得
						if numThick == 1:
							outerThick.append(getAttributeCheckFloat(child.getAttribute('thick')))
						else:
							for num in range(numThick):
								outerThick.append(getAttributeCheckFloat(child.getAttribute('thick'+str(num+1))))

						outerDensity = getAttributeCheckFloat(child.getAttribute('density'))
						# elementの取得
						outerElement = GetElement(child,'outer')
					if child.tagName == 'gas':
						# 属性：pressure, temperature取得
						gasPressure = getAttributeCheckFloat(child.getAttribute('pressure'))
						gasTemperture = getAttributeCheckFloat(child.getAttribute('temperature'))
						# elementの取得
						gasElement = GetElement(child,'gas')

			detec = EffPrmInfoRead(name, size, outerThick, outerDensity, gasPressure, gasTemperture, outerElement, gasElement)
			self.detecList.append(detec)

		return

	##############################
	def GetReadEfficiencyParam(self, name="SUS304-3He"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoRead)検出器の構成情報
		"""
		for readDetecn in self.detecList:
			if readDetecn.name == name:
				return readDetecn

		return None


##############################
def GetElement(node,strType):
	element = {}
	for child in node.childNodes:
		if child.nodeType == Node.ELEMENT_NODE:
			if child.tagName == 'element':
				name = getAttributeCheckStr(child.getAttribute('name'))
				ratio = 0.0
				if strType == 'gas':
					ratio = getAttributeCheckFloat(child.getAttribute('ratio'))
				if strType == 'outer':
					ratio = getAttributeCheckFloat(child.getAttribute('wt-ratio'))

				element[name] = ratio
	return element

##############################
def getAttributeCheckStr(obj):
	try:
		ret = str(obj)
	except ValueError:
		ret = ""

	return ret

##############################
def getAttributeCheckFloat(obj):
	try:
		ret = float(obj)
	except ValueError:
		ret = 0.0

	return ret

###############################################################################
# 検出器効率計算クラス
###############################################################################
class CalcEfficiencyParam:
	"""
		検出器効率の計算を行う
		属性：
		detecList 				:(EffPrmInfoCalc)[]計算済みの検出器の構成情報リスト
		readEffPrm					: ReadEfficiencyParam	検出器構成情報ファイル読み込みインスタンス
		adbi					: AtomDBTaikan	大観用原子情報DBインスタンス
	"""

	##############################
	def __init__(self, strEffPrmInfo = 'psdinfo', strSize = 'radius', strOuter = 'cylinder', numThick = 1):
		"""
		初期化処理(コンストラクタ)
		属性の初期化を行う。
		@param	strEffPrmInfo	(str)検出器効率パラメータ情報のタグ名
										(psd:psdinfo	monitor:monitorinfo)
		@param	strSize			(str)検出器サイズのタグ名
										(psd:radius		monitor:width)
		@param	strOuter		(str)外殻のタグ名
										(psd:cylinder	monitor:shell)
		@param	numThick		(integer)thickの数
										(psd:1			monitor:2)
		"""
		self.detecList = []

		# ReadEfficiencyParam読み込み
		readEffPrm = ReadEfficiencyParam(strEffPrmInfo,strSize,strOuter,numThick)
		self.readEffPrm = readEffPrm

		#AtomDBTaikanの初期化
		adbi = ADB.AtomDBTaikanClass()
		adbi.ReadDBFile()
		self.adbi = adbi

		#計算
		self.__CalcParam()


	##############################
	def GetCalcEfficiencyParam(self, name="SUS304-3He"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoCalc)計算済みの検出器の構成情報
		"""
		for calcDetecn in self.detecList:
			if calcDetecn.name == name:
				return calcDetecn

		return None

	##############################
	def GetReadEfficiencyParam(self, name="SUS304-3He"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoRead)検出器の構成情報
		"""
		return self.readEffPrm.GetReadEfficiencyParam(name)

	##############################
	def __CalcParam(self):
		"""
		パラメータの計算
		"""
		for readDetecn in self.readEffPrm.detecList:
			name				= readDetecn.name				# name
			size				= readDetecn.size				# PSDの半径		[cm]
			outerThick			= readDetecn.outerThick			# 外殻肉厚リスト		[cm]

			# 外殻成分
			outerElement			= list(readDetecn.outerElement.keys())		# 外殻成分元素
			outerElementWtRatio		= list(readDetecn.outerElement.values())		# 外殻成分構成比(重量比)
			outerDensity			= readDetecn.outerDensity				# 外殻密度	[g/cm^3]
			# ガス成分
			gasElement			= list(readDetecn.gasElement.keys())			# ガス成分元素リスト
			gasElementRatio		= list(readDetecn.gasElement.values())		# ガス成分構成比リスト
			if readDetecn.gasTemperture == 0:
				gasNumberDensity = 0.0
			else:
				gasNumberDensity	= (readDetecn.gasPressure * N_A) / (R_GAS_CONST * readDetecn.gasTemperture * 1000)
																# ガス数密度[個/cm^3]

			# AtomDBTaikanから値を取得する[barn]

			outerElementAbs			= []					# 外殻成分吸収断面積リスト[barn]
			outerElementScatt		= []					# 外殻成分散乱断面積リスト[barn]
			outerElementMolWeight	= []					# 外殻成分分子量リスト	[g/mol]

			gasElementAbs		= []							# ガス成分吸収断面積リスト[barn]

			for elem in outerElement:
				# 外殻成分吸収断面積リスト[barn]
				x=[]
				ret = self.adbi.GetValue(elem, ADB.DEF_ATOM_DB_ABSXS_TAG,x)
				fx=float(x[0])
				outerElementAbs.append(fx)
				# 外殻成分散乱断面積リスト[barn]
				x=[]
				self.adbi.GetValue(elem, ADB.DEF_ATOM_DB_SCATTXS_TAG,x)
				fx=float(x[0])
				outerElementScatt.append(fx)
				# 外殻成分分子量リスト	[g/mol]
				x=[]
				self.adbi.GetValue(elem, ADB.DEF_ATOM_DB_MOLWEIGHT_TAG,x)
				fx=float(x[0])
				outerElementMolWeight.append(fx)

			#print outerElementAbs
			#print outerElementScatt
			#print outerElementMolWeight

			for elem in gasElement:
				# ガス成分吸収断面積リスト[barn]
				x=[]
				self.adbi.GetValue(elem, ADB.DEF_ATOM_DB_ABSXS_TAG,x)
				fx=float(x[0])
				gasElementAbs.append(fx)

			#print gasElementAbs

			# 各値を計算する
			# 外殻成分
			outerMolWeight			= CalcElementMolWeight(outerElementMolWeight, outerElementWtRatio)
																	# 外殻全分子量	[g/mol]

			outerElementRatio = [outerMolWeight*outerElementWtRatio[x]/outerElementMolWeight[x] for x in range(len(outerElementMolWeight))]
																	# 重量比→モル比
			#print outerElementWtRatio
			#print outerElementRatio

			outerAbs			= CalcElementAbs(outerElementAbs, outerElementRatio, 'outer')
																	# 外殻全吸収断面積[cm^2]
			outerScatt		= CalcElementScatt(outerElementScatt, outerElementRatio)
																	# 外殻全散乱断面積[cm^2]

			if outerMolWeight == 0:
				outerNumberDensity	= 0.0
			else:
				outerNumberDensity	= (outerDensity * N_A)/outerMolWeight
																	# 外殻数密度		[個/cm^3]
			# ガス成分
			gasAbs			= CalcElementAbs(gasElementAbs, gasElementRatio, 'gas')
																# ガス全吸収断面積[cm^2]
			detec = EffPrmInfoCalc(name, size, outerThick, outerAbs, outerScatt, outerNumberDensity, gasAbs, gasNumberDensity)
			self.detecList.append(detec)
		return

###############################################################################
# PSD検出器効率計算クラス
###############################################################################
class CalcPsdEfficiencyParam(CalcEfficiencyParam):
	"""
		PSD検出器効率の取得と計算を行う
	"""
	##############################
	def __init__(self):
		CalcEfficiencyParam.__init__(self,'psdinfo', 'radius', 'cylinder', 1)

	##############################
	def GetCalcPsdEffParam(self, name="SUS304-3He"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoCalc)計算済みの検出器の構成情報
		"""
		return CalcEfficiencyParam.GetCalcEfficiencyParam(self,name)

	##############################
	def GetReadPsdEffParam(self, name="SUS304-3He"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoRead)検出器の構成情報
		"""
		return CalcEfficiencyParam.GetReadEfficiencyParam(self,name)

###############################################################################
# MONITOR検出器効率計算クラス
###############################################################################
class CalcMonitorEfficiencyParam(CalcEfficiencyParam):
	"""
		MONITOR検出器効率の取得と計算を行う
	"""
	##############################
	def __init__(self):
		CalcEfficiencyParam.__init__(self,'monitorinfo', 'width', 'shell', 2)

	##############################
	def GetCalcMonitorEffParam(self, name="MONITOR-A5083-1"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoCalc)計算済みの検出器の構成情報
		"""
		return CalcEfficiencyParam.GetCalcEfficiencyParam(self,name)

	##############################
	def GetReadMonitorEffParam(self, name="MONITOR-A5083-1"):
		"""
		@param	name	(str)検出器の名前
		@retval			(EffPrmInfoRead)検出器の構成情報
		"""
		return CalcEfficiencyParam.GetReadEfficiencyParam(self,name)


##############################
def CalcSumProduct(List1, List2):
	"""
	リストの対応する要素の積を合計する
	@param List1	(float)[]	リスト1
	@param List2	(float)[]	リスト2
	@retval			(float)	全吸収断面積
	"""
	retval = 0.0

	for x in range(len(List1)):
		retval = retval + List1[x] * List2[x]

	return retval

##############################
def CalcElementMolWeight(molWeightList, wtRatioList):
	"""
	全分子量の計算
	@param molWeightList	(float)[]	分子量リスト
	@param wtRatioList		(float)[]	構成比リスト(重量比)
	@retval					(float)	全吸収断面積
	"""
	invmolWeightList = [1/x for x in molWeightList]

	retval = CalcSumProduct(invmolWeightList, wtRatioList)
	retval = 1/retval

	return retval

##############################
def CalcElementAbs(absList, ratioList, strType):
	"""
	全吸収断面積の計算
	@param absList		(float)[]	吸収断面積リスト[barn]
	@param ratioList	(float)[]	構成比リスト
	@param strType		(str)[]	外殻(outer)orガス(gas)の文字列
	@retval				(float)	全吸収断面積	[cm^2]
	"""
	retval = 0.0
	if 'outer' == strType:
		print("DEBUG outer")
		retval = CalcSumProduct(absList, ratioList)/sum(ratioList)
	elif 'gas'  == strType:
		print("DEBUG gas")
		retval = CalcSumProduct(absList, ratioList)

	retval = retval / ABS_WAVE_LENGTH
	# [barn] → [cm^2]
	retval = retval * (1.0e-24)

	return retval

##############################
def CalcElementScatt(scattList, ratioList):
	"""
	全散乱断面積の計算
	@param scattList	(float)[]	散乱断面積リスト[barn]
	@param ratioList	(float)[]	構成比リスト
	@retval				(float)	全散乱断面積	[cm^2]
	"""
	retval = CalcSumProduct(scattList, ratioList)/sum(ratioList)
	# [barn] → [cm^2]
	retval = retval * (1.0e-24)

	return retval




#! /usr/bin/env python
from __future__ import print_function
import os
import sys
import DR
import Cmm

#####################
# modify here
####################
XtalParam = "XtalParam.xml"
savePathDir = "./data"
#########################

listFileName=""

### Check Arguments
argv = sys.argv
if len(argv)<2 or len(argv)>4:
    print("Usage::")
    print("        $ ./{} <filename of runlist> [<output dir> [<XtalParam>]]".format(argv[0]))
    print("        ")
    print("        $ python {} <filename of runlist> [<output dir> [<XtalParam>]]".format(argv[0]))
    sys.exit(0)

### Check and set run list file
### <filename> <angle>
if len(argv)>1:
    if os.path.exists(argv[1]):
        listFileName=argv[1]
        print("RunList=[]".format(listFileName))
    else:
        raise UserWarning("Not found file= {}".format(argv[1]))

### Check and set output directory
if len(argv)>2 and os.path.exists(argv[2]):
    savePathDir = argv[2]
else:
    raise UserWarning("Not found directory = {}".format(argv[2]))

### Check and set XtalParam file
if len(argv)>3 and os.path.exists(argv[3]):
    XtalParam = argv[3]
else:
    raise UserWarning("Not found XtalParam = {}".format(argv[3]))

### If output directory does not existed
if not os.path.exists(savePathDir):
    print(" savePathDir = {} is not found. Making new ".format(savePathDir))
    try:
        os.makedirs(savePathDir)
    except:
        raise UserWarning("Cannot make new folder(" + savePathDir +")")

### Main loop for conversion
fr = open( listFileName, "r" )
while(True):
    a_line = fr.readline()
    if a_line=="":
        break
    params = a_line.split(" ")
    if len(params)!=2:
        raise UserWarning("Cannot read this format (" + a_line +")")
    root_path = os.path.dirname(params[0])
    if root_path=="":
        root_path = "./"
    Filename = os.path.basename(params[0])
    PhiValue = float( params[1] )
    savePath = os.path.join( savePathDir, "%08.3f.vbin"%(PhiValue))
    print("==============[ %s, %8.3f, saved to %s ]" % (Filename,PhiValue,savePath))
    DAT = Cmm.LoadDataFromDump(path=root_path,filename=Filename)
    Cmm.VisContMOutputFileByPhi(DAT, XtalParam, PhiValue, savePath)
    del DAT

fr.close()
